@@ -10,9 +10,7 @@
 
 
 import glob
-import multiprocessing
 import os
-import pickle
 import shlex
 import shutil
 import signal
@@ -23,10 +21,11 @@ import zipfile
 
 from threading import Timer
 
+import multiprocess
 import psutil
 
-from codechecker_analyzer import env
 from codechecker_common.logger import get_logger
+from codechecker_common.review_status_handler import ReviewStatusHandler
 
 from codechecker_statistics_collector.collectors.special_return_value import \
     SpecialReturnValueCollector
@@ -60,8 +59,7 @@ def print_analyzer_statistic_summary(metadata_analyzers, status, msg=None):
             LOG.info("  %s: %s", analyzer_type, res)
 
 
-def worker_result_handler(results, metadata_tool, output_path,
-                          analyzer_binaries):
+def worker_result_handler(results, metadata_tool, output_path):
     """ Print the analysis summary. """
     skipped_num = 0
     reanalyzed_num = 0
@@ -169,7 +167,7 @@ def is_ctu_active(source_analyzer):
         source_analyzer.is_ctu_enabled()
 
 
-def prepare_check(action, analyzer_config, output_dir, checker_labels,
+def prepare_check(action, analyzer_config, output_dir,
                   skip_handlers, statistics_data, disable_ctu=False):
     """ Construct the source analyzer and result handler. """
     # Create a source analyzer.
@@ -209,7 +207,6 @@ def prepare_check(action, analyzer_config, output_dir, checker_labels,
     # which only returns metadata, but can't process the results.
     rh = source_analyzer.construct_result_handler(action,
                                                   output_dir,
-                                                  checker_labels,
                                                   skip_handlers)
 
     # NOTICE!
@@ -221,8 +218,11 @@ def prepare_check(action, analyzer_config, output_dir, checker_labels,
     return source_analyzer, rh
 
 
-def handle_success(rh, result_file, result_base, skip_handlers,
-                   capture_analysis_output, success_dir):
+def handle_success(
+    rh, result_file, result_base, skip_handlers,
+    rs_handler: ReviewStatusHandler,
+    capture_analysis_output, success_dir
+):
     """
     Result postprocessing is required if the analysis was
     successful (mainly clang tidy output conversion is done).
@@ -233,7 +233,7 @@ def handle_success(rh, result_file, result_base, skip_handlers,
         save_output(os.path.join(success_dir, result_base),
                     rh.analyzer_stdout, rh.analyzer_stderr)
 
-    rh.postprocess_result(skip_handlers)
+    rh.postprocess_result(skip_handlers, rs_handler)
 
     # Generated reports will be handled separately at store.
 
@@ -320,7 +320,8 @@ def handle_reproducer(source_analyzer, rh, zip_file, actions_map):
 
 
 def handle_failure(
-    source_analyzer, rh, zip_file, result_base, actions_map, skip_handlers
+    source_analyzer, rh, zip_file, result_base, actions_map, skip_handlers,
+    rs_handler: ReviewStatusHandler
 ):
     """
     If the analysis fails a debug zip is packed together which contains
@@ -333,9 +334,9 @@ def handle_failure(
     # from the standard output by this postprocess phase so we can present them
     # as CodeChecker reports.
     checks = source_analyzer.config_handler.checks()
-    state = checks.get('clang-diagnostic-error', (CheckerState.default, ''))[0]
-    if state != CheckerState.disabled:
-        rh.postprocess_result(skip_handlers)
+    state = checks.get('clang-diagnostic-error', (CheckerState.enabled, ''))[0]
+    if state == CheckerState.enabled:
+        rh.postprocess_result(skip_handlers, rs_handler)
 
     # Remove files that successfully analyzed earlier on.
     plist_file = result_base + ".plist"
@@ -488,10 +489,10 @@ def check(check_data):
 
     skiplist handler is None if no skip file was configured.
     """
-    actions_map, action, context, analyzer_config, \
-        output_dir, skip_handlers, quiet_output_on_stdout, \
+    actions_map, action, analyzer_config, \
+        output_dir, skip_handlers, rs_handler, quiet_output_on_stdout, \
         capture_analysis_output, generate_reproducer, analysis_timeout, \
-        analyzer_environment, ctu_reanalyze_on_failure, \
+        ctu_reanalyze_on_failure, \
         output_dirs, statistics_data = check_data
 
     failed_dir = output_dirs["failed"]
@@ -509,7 +510,7 @@ def check(check_data):
             raise Exception("Analyzer configuration is missing.")
 
         source_analyzer, rh = prepare_check(action, analyzer_config,
-                                            output_dir, context.checker_labels,
+                                            output_dir,
                                             skip_handlers, statistics_data)
 
         reanalyzed = os.path.exists(rh.analyzer_result_file)
@@ -545,20 +546,8 @@ def check(check_data):
 
         result_file_exists = os.path.exists(rh.analyzer_result_file)
 
-        # FIXME: cppcheck (and other analyzers)
-        # need the original env when invoked
-        # we should handle this in a generic way.
-        if "cppcheck" in os.path.basename(analyzer_cmd[0]):
-            original_env_file = os.environ.get(
-                'CODECHECKER_ORIGINAL_BUILD_ENV')
-            if original_env_file:
-                with open(original_env_file, 'rb') as env_file:
-                    analyzer_environment = \
-                        pickle.load(env_file, encoding='utf-8')
-
         # Fills up the result handler with the analyzer information.
-        source_analyzer.analyze(analyzer_cmd, rh, analyzer_environment,
-                                __create_timeout)
+        source_analyzer.analyze(analyzer_cmd, rh, __create_timeout)
 
         # If execution reaches this line, the analyzer process has quit.
         if timeout_cleanup[0]():
@@ -619,12 +608,13 @@ def check(check_data):
 
             if success:
                 handle_success(rh, result_file, result_base,
-                               skip_handlers, capture_analysis_output,
-                               success_dir)
+                               skip_handlers, rs_handler,
+                               capture_analysis_output, success_dir)
             elif not generate_reproducer:
                 handle_failure(source_analyzer, rh,
                                os.path.join(failed_dir, zip_file),
-                               result_base, actions_map, skip_handlers)
+                               result_base, actions_map, skip_handlers,
+                               rs_handler)
 
         if rh.analyzer_returncode == 0:
             handle_analysis_result(success=True)
@@ -652,7 +642,7 @@ def check(check_data):
                 # Try to reanalyze with CTU disabled.
                 source_analyzer, rh = \
                     prepare_check(action, analyzer_config,
-                                  output_dir, context.checker_labels,
+                                  output_dir,
                                   skip_handlers, statistics_data,
                                   True)
                 reanalyzed = os.path.exists(rh.analyzer_result_file)
@@ -662,9 +652,7 @@ def check(check_data):
 
                 # Fills up the result handler with
                 # the analyzer information.
-                source_analyzer.analyze(analyzer_cmd,
-                                        rh,
-                                        analyzer_environment)
+                source_analyzer.analyze(analyzer_cmd, rh)
 
                 return_codes = rh.analyzer_returncode
                 if rh.analyzer_returncode == 0:
@@ -733,8 +721,9 @@ def skip_cpp(compile_actions, skip_handlers):
     return analyze, skip
 
 
-def start_workers(actions_map, actions, context, analyzer_config_map,
-                  jobs, output_path, skip_handlers, metadata_tool,
+def start_workers(actions_map, actions, analyzer_config_map,
+                  jobs, output_path, skip_handlers,
+                  rs_handler: ReviewStatusHandler, metadata_tool,
                   quiet_analyze, capture_analysis_output, generate_reproducer,
                   timeout, ctu_reanalyze_on_failure, statistics_data, manager,
                   compile_cmd_count):
@@ -742,24 +731,29 @@ def start_workers(actions_map, actions, context, analyzer_config_map,
     Start the workers in the process pool.
     For every build action there is worker which makes the analysis.
     """
+    # pylint: disable=no-member multiprocess module members.
 
     # Handle SIGINT to stop this script running.
     def signal_handler(signum, frame):
         try:
             pool.terminate()
+            pool.join()
             manager.shutdown()
+        except Exception as e:
+            LOG.error("Failed to clean up after the pool!:\n")
+            LOG.error(e)
+            raise
         finally:
             sys.exit(128 + signum)
 
-    signal.signal(signal.SIGINT, signal_handler)
     actions, skipped_actions = skip_cpp(actions, skip_handlers)
     # Start checking parallel.
-    checked_var = multiprocessing.Value('i', 1)
-    actions_num = multiprocessing.Value('i', len(actions))
-    pool = multiprocessing.Pool(jobs,
-                                initializer=init_worker,
-                                initargs=(checked_var,
-                                          actions_num))
+    checked_var = multiprocess.Value('i', 1)
+    actions_num = multiprocess.Value('i', len(actions))
+    pool = multiprocess.Pool(jobs,
+                             initializer=init_worker,
+                             initargs=(checked_var, actions_num))
+    signal.signal(signal.SIGINT, signal_handler)
 
     # If the analysis has failed, we help debugging.
     failed_dir = os.path.join(output_path, "failed")
@@ -791,21 +785,16 @@ def start_workers(actions_map, actions, context, analyzer_config_map,
                    'reproducer': reproducer_dir,
                    'ctu_connections': ctu_connections_dir}
 
-    # Construct analyzer env.
-    analyzer_environment = env.extend(context.path_env_extra,
-                                      context.ld_lib_path_extra)
-
     analyzed_actions = [(actions_map,
                          build_action,
-                         context,
                          analyzer_config_map.get(build_action.analyzer_type),
                          output_path,
                          skip_handlers,
+                         rs_handler,
                          quiet_analyze,
                          capture_analysis_output,
                          generate_reproducer,
                          timeout,
-                         analyzer_environment,
                          ctu_reanalyze_on_failure,
                          output_dirs,
                          statistics_data)
@@ -829,8 +818,7 @@ def start_workers(actions_map, actions, context, analyzer_config_map,
                            analyzed_actions,
                            1,
                            callback=lambda results: worker_result_handler(
-                               results, metadata_tool, output_path,
-                               context.analyzer_binaries)
+                               results, metadata_tool, output_path)
                            ).get(timeout)
 
             pool.close()
