@@ -9,27 +9,31 @@
 Clang Static Analyzer related functions.
 """
 
-
 import os
+import plistlib
 import re
 import shlex
 import subprocess
 
-from typing import Dict, List
+from typing import List
 
 from codechecker_common.logger import get_logger
 
-from codechecker_analyzer import env
+from codechecker_analyzer import analyzer_context, env
+from codechecker_statistics_collector.collectors.special_return_value import \
+    SpecialReturnValueCollector
+from codechecker_statistics_collector.collectors.return_value import \
+    ReturnValueCollector
 
 from .. import analyzer_base
 from ..config_handler import CheckerState
 from ..flag import has_flag
 from ..flag import prepend_all
 
-from . import clang_options
 from . import config_handler
 from . import ctu_triple_arch
 from . import version
+from .ctu_autodetection import CTUAutodetection
 from .result_handler import ClangSAResultHandler
 
 LOG = get_logger('analyzer')
@@ -37,8 +41,7 @@ LOG = get_logger('analyzer')
 
 def parse_clang_help_page(
     command: List[str],
-    start_label: str,
-    environ: Dict[str, str]
+    start_label: str
 ) -> List[str]:
     """
     Parse the clang help page starting from a specific label.
@@ -48,7 +51,7 @@ def parse_clang_help_page(
         help_page = subprocess.check_output(
             command,
             stderr=subprocess.STDOUT,
-            env=environ,
+            env=analyzer_context.get_context().analyzer_env,
             universal_newlines=True,
             encoding="utf-8",
             errors="ignore")
@@ -96,16 +99,117 @@ def parse_clang_help_page(
     return res
 
 
+def _is_user_disabled_checker(checker, ordered_checkers):
+    """
+    This function returns True if the given checker is disabled by the user
+    explicitly by a --disable flag.
+    """
+    if not ordered_checkers:
+        return False
+
+    disabled_checkers = (c for c, enabled in ordered_checkers if not enabled)
+
+    return any(checker.startswith(c) for c in disabled_checkers)
+
+
 class ClangSA(analyzer_base.SourceAnalyzer):
     """
     Constructs clang static analyzer commands.
     """
     ANALYZER_NAME = 'clangsa'
 
+    __ctu_autodetection = None
+
     def __init__(self, cfg_handler, buildaction):
         super(ClangSA, self).__init__(cfg_handler, buildaction)
         self.__disable_ctu = False
         self.__checker_configs = []
+        self.__disabled_checkers = []
+
+    @classmethod
+    def analyzer_binary(cls):
+        """
+        Return the path of the analyzer binary.
+        """
+        return analyzer_context.get_context() \
+            .analyzer_binaries[cls.ANALYZER_NAME]
+
+    @classmethod
+    def analyzer_plugins(cls) -> List[str]:
+        """
+        Return the list of .so file paths which contain checker plugins to
+        ClangSA.
+        """
+        plugin_dir = analyzer_context.get_context().checker_plugin
+
+        clangsa_plugin_dir = env.get_clangsa_plugin_dir()
+        is_analyzer_from_path = env.is_analyzer_from_path()
+        if is_analyzer_from_path:
+            if not clangsa_plugin_dir:
+                return []
+
+            # If the CC_ANALYZERS_FROM_PATH and CC_CLANGSA_PLUGIN_DIR
+            # environment variables are set we will use this value as the
+            # plugin directory.
+            plugin_dir = clangsa_plugin_dir
+
+        if not plugin_dir or not os.path.exists(plugin_dir):
+            return []
+
+        return [os.path.join(plugin_dir, f)
+                for f in os.listdir(plugin_dir)
+                if os.path.isfile(os.path.join(plugin_dir, f))
+                and f.endswith(".so")]
+
+    @classmethod
+    def __add_plugin_load_flags(cls, analyzer_cmd: List[str]):
+        """
+        ClangSA can be extended with checker plugins. This function extends a
+        clang command with these plugins.
+        """
+        for plugin in ClangSA.analyzer_plugins():
+            analyzer_cmd.extend(["-load", plugin])
+
+    @classmethod
+    def get_binary_version(self, environ, details=False) -> str:
+        # No need to LOG here, we will emit a warning later anyway.
+        if not self.analyzer_binary():
+            return None
+
+        if details:
+            version = [self.analyzer_binary(), '--version']
+        else:
+            version = [self.analyzer_binary(), '-dumpversion']
+        try:
+            output = subprocess.check_output(version,
+                                             env=environ,
+                                             universal_newlines=True,
+                                             encoding="utf-8",
+                                             errors="ignore")
+            return output.strip()
+        except (subprocess.CalledProcessError, OSError) as oerr:
+            LOG.warning("Failed to get analyzer version: %s",
+                        ' '.join(version))
+            LOG.warning(oerr)
+
+        return None
+
+    @classmethod
+    def version_info(cls):
+        return version.get(cls.analyzer_binary())
+
+    @classmethod
+    def ctu_capability(cls):
+        """
+        Return a CTUAutodetection object which describes the availability of
+        CTU feature with some additional CTU-related info.
+        """
+        if not cls.__ctu_autodetection:
+            cls.__ctu_autodetection = CTUAutodetection(
+                cls.analyzer_binary(),
+                analyzer_context.get_context().analyzer_env)
+
+        return cls.__ctu_autodetection
 
     def is_ctu_available(self):
         """
@@ -144,40 +248,107 @@ class ClangSA(analyzer_base.SourceAnalyzer):
     @classmethod
     def get_analyzer_checkers(
         cls,
-        cfg_handler: config_handler.ClangSAConfigHandler,
-        environ: Dict[str, str],
         alpha: bool = True,
         debug: bool = False
     ) -> List[str]:
-        """Return the list of the supported checkers."""
-        checker_list_args = clang_options.get_analyzer_checkers_cmd(
-            cfg_handler,
-            alpha=alpha,
-            debug=debug)
-        return parse_clang_help_page(checker_list_args, 'CHECKERS:', environ)
+        """
+        Return the list of the supported checkers.
+
+        Before clang9 alpha and debug checkers were printed by default. Since
+        clang9 there are extra arguments to print the additional checkers.
+        """
+        command = [cls.analyzer_binary(), "-cc1"]
+
+        cls.__add_plugin_load_flags(command)
+
+        command.append("-analyzer-checker-help")
+
+        # The clang compiler on OSX is a few
+        # relases older than the open source clang release.
+        # The new checker help printig flags are not available there yet.
+        # If the OSX clang will be updated to based on clang v8
+        # this early return can be removed.
+        version_info = cls.version_info()
+        if version_info and version_info.vendor == "clang":
+            if alpha and version_info.major_version >= 9:
+                command.append("-analyzer-checker-help-alpha")
+
+            if debug and version_info.major_version >= 9:
+                command.append("-analyzer-checker-help-developer")
+
+        return parse_clang_help_page(command, 'CHECKERS:')
 
     @classmethod
-    def get_checker_config(
-        cls,
-        cfg_handler: config_handler.ClangSAConfigHandler,
-        environ: Dict[str, str]
-    ) -> List[str]:
-        """Return the list of checker config options."""
-        checker_config_args = clang_options.get_checker_config_cmd(
-            cfg_handler,
-            alpha=True)
-        return parse_clang_help_page(checker_config_args, 'OPTIONS:', environ)
+    def get_checker_config(cls) -> List[str]:
+        """
+        Return the list of checker config options.
+
+        Before clang9 alpha and debug checkers were printed by default. Since
+        clang9 there are extra arguments to print the additional checkers.
+        """
+        command = [cls.analyzer_binary(), "-cc1"]
+
+        cls.__add_plugin_load_flags(command)
+
+        command.append("-analyzer-checker-option-help")
+
+        version_info = ClangSA.version_info()
+        if version_info.vendor == "clang":
+            if version_info.major_version >= 9:
+                command.append("-analyzer-checker-option-help-alpha")
+
+            if version_info.major_version >= 9:
+                command.append("-analyzer-checker-option-help-developer")
+
+        return parse_clang_help_page(command, 'OPTIONS:')
 
     @classmethod
-    def get_analyzer_config(
-        cls,
-        cfg_handler: config_handler.ClangSAConfigHandler,
-        environ: Dict[str, str]
-    ) -> List[str]:
+    def get_analyzer_config(cls) -> List[str]:
         """Return the list of analyzer config options."""
-        analyzer_config_args = clang_options.get_analyzer_config_cmd(
-            cfg_handler)
-        return parse_clang_help_page(analyzer_config_args, 'OPTIONS:', environ)
+        command = [cls.analyzer_binary(), "-cc1"]
+
+        cls.__add_plugin_load_flags(command)
+
+        command.append("-analyzer-config-help")
+
+        return parse_clang_help_page(command, 'OPTIONS:')
+
+    def post_analyze(self, result_handler):
+        """
+        Disabled checkers are not actually disabled during analysis, because
+        they might rely on each other under the hood. The disabled checkers'
+        reports are removed in this post-processing step.
+        """
+        try:
+            if not os.path.isfile(result_handler.analyzer_result_file):
+                # This check has the same race-condition reason as the
+                # exception, so see its description below.
+                return
+
+            with open(result_handler.analyzer_result_file, 'rb') as f:
+                plist = plistlib.load(f)
+        except plistlib.InvalidFileException:
+            # It may happen that a compilation database contains a build action
+            # multiple times (because it is compiled for several target
+            # architectures), or at least they differ in a so minor part that
+            # the same .plist file belongs to them. (For further details see
+            # analyzer_action_str() in ResultHandler about the strange behavior
+            # of make 4.3 in its -o flag.)
+            # If this happens then the analysis of the same build action is
+            # analyzed in parallel several times, so the same output .plist
+            # file is changed by several threads. This may result an invalid
+            # .plist which fails plistlib.load(). This is not a big problem,
+            # because the second thread will also execute this post-processing
+            # and it happens rarely anyways. test_compile_uniqueing() fails
+            # undeterministically without this.
+            return
+
+        plist['diagnostics'] = list(filter(
+            lambda diag: diag['check_name'] not in self.__disabled_checkers,
+            plist.get('diagnostics', [])))
+
+        with open(result_handler.analyzer_result_file, 'wb') as f:
+            plistlib.dump(plist, f)
 
     def construct_analyzer_cmd(self, result_handler):
         """
@@ -192,11 +363,11 @@ class ClangSA(analyzer_base.SourceAnalyzer):
             # Checker order matters.
             config = self.config_handler
 
-            analyzer_cmd = [config.analyzer_binary, '--analyze',
+            analyzer_cmd = [ClangSA.analyzer_binary(), '--analyze',
                             # Do not warn about the unused gcc/g++ arguments.
                             '-Qunused-arguments']
 
-            for plugin in config.analyzer_plugins:
+            for plugin in ClangSA.analyzer_plugins():
                 analyzer_cmd.extend(["-Xclang", "-plugin",
                                      "-Xclang", "checkercfg",
                                      "-Xclang", "-load",
@@ -230,26 +401,27 @@ class ClangSA(analyzer_base.SourceAnalyzer):
                     ['-Xclang', '-analyzer-config', '-Xclang', cfg])
 
             # Config handler stores which checkers are enabled or disabled.
-            disabled_checkers = []
+            self.__disabled_checkers = []
             enabled_checkers = []
             for checker_name, value in config.checks().items():
                 state, _ = value
                 if state == CheckerState.enabled:
                     enabled_checkers.append(checker_name)
                 elif state == CheckerState.disabled:
-                    disabled_checkers.append(checker_name)
+                    self.__disabled_checkers.append(checker_name)
 
             if enabled_checkers:
                 analyzer_cmd.extend(['-Xclang',
                                      '-analyzer-checker=' +
                                      ','.join(enabled_checkers)])
-            if disabled_checkers:
-                analyzer_cmd.extend(['-Xclang',
-                                     '-analyzer-disable-checker=' +
-                                     ','.join(disabled_checkers)])
             # Enable aggressive-binary-operation-simplification option.
-            analyzer_cmd.extend(
-                clang_options.get_abos_options(config.version_info))
+            version_info = ClangSA.version_info()
+            if version_info and version_info.major_version >= 8:
+                analyzer_cmd.extend([
+                    '-Xclang',
+                    '-analyzer-config',
+                    '-Xclang',
+                    'aggressive-binary-operation-simplification=true'])
 
             # Enable the z3 solver backend.
             if config.enable_z3:
@@ -267,7 +439,8 @@ class ClangSA(analyzer_base.SourceAnalyzer):
                      'experimental-enable-naive-ctu-analysis=true',
                      '-Xclang', '-analyzer-config', '-Xclang',
                      'ctu-dir=' + self.get_ctu_dir()])
-                ctu_display_progress = config.ctu_capability.display_progress
+                ctu_display_progress = \
+                    ClangSA.ctu_capability().display_progress
                 if ctu_display_progress:
                     analyzer_cmd.extend(ctu_display_progress)
 
@@ -289,7 +462,7 @@ class ClangSA(analyzer_base.SourceAnalyzer):
 
             if not has_flag('-arch', analyzer_cmd) and \
                     self.buildaction.arch != "":
-                analyzer_cmd.extend(["-arch ", self.buildaction.arch])
+                analyzer_cmd.extend(["-arch", self.buildaction.arch])
 
             if not has_flag('-std', analyzer_cmd) and \
                     self.buildaction.compiler_standard != "":
@@ -316,11 +489,9 @@ class ClangSA(analyzer_base.SourceAnalyzer):
         Returns the path of the ctu directory (containing the triple).
         """
         config = self.config_handler
-        environ = env.extend(config.path_env_extra,
-                             config.ld_lib_path_extra)
         triple_arch = ctu_triple_arch.get_triple_arch(self.buildaction,
                                                       self.source_file,
-                                                      config, environ)
+                                                      config)
         ctu_dir = os.path.join(config.ctu_dir, triple_arch)
         return ctu_dir
 
@@ -413,36 +584,31 @@ class ClangSA(analyzer_base.SourceAnalyzer):
         return clang
 
     @classmethod
-    def version_compatible(cls, configured_binary, environ):
+    def is_binary_version_incompatible(cls, environ):
         """
-        Check the version compatibility of the given analyzer binary.
+        We support pretty much every ClangSA version.
         """
-        return True
+        return None
 
     def construct_result_handler(self, buildaction, report_output,
-                                 checker_labels, skiplist_handler):
+                                 skiplist_handler):
         """
         See base class for docs.
         """
         res_handler = ClangSAResultHandler(buildaction, report_output,
                                            self.config_handler.report_hash)
 
-        res_handler.checker_labels = checker_labels
         res_handler.skiplist_handler = skiplist_handler
 
         return res_handler
 
     @classmethod
-    def construct_config_handler(cls, args, context):
+    def construct_config_handler(cls, args):
 
-        environ = env.extend(context.path_env_extra,
-                             context.ld_lib_path_extra)
+        context = analyzer_context.get_context()
+        environ = context.analyzer_env
 
         handler = config_handler.ClangSAConfigHandler(environ)
-        handler.analyzer_plugins_dir = context.checker_plugin
-        handler.analyzer_binary = context.analyzer_binaries.get(
-            cls.ANALYZER_NAME)
-        handler.version_info = version.get(handler.analyzer_binary, environ)
 
         handler.report_hash = args.report_hash \
             if 'report_hash' in args else None
@@ -458,9 +624,6 @@ class ClangSA(analyzer_base.SourceAnalyzer):
             handler.ctu_on_demand = \
                 'ctu_ast_mode' in args and \
                 args.ctu_ast_mode == 'parse-on-demand'
-            handler.log_file = args.logfile
-            handler.path_env_extra = context.path_env_extra
-            handler.ld_lib_path_extra = context.ld_lib_path_extra
 
         try:
             with open(args.clangsa_args_cfg_file, 'r', encoding='utf8',
@@ -477,7 +640,7 @@ class ClangSA(analyzer_base.SourceAnalyzer):
             # No clangsa arguments file was given in the command line.
             LOG.debug_analyzer(aerr)
 
-        checkers = ClangSA.get_analyzer_checkers(handler, environ)
+        checkers = ClangSA.get_analyzer_checkers()
 
         try:
             cmdline_checkers = args.ordered_checkers
@@ -487,23 +650,54 @@ class ClangSA(analyzer_base.SourceAnalyzer):
             cmdline_checkers = []
 
         handler.initialize_checkers(
-            context,
             checkers,
             cmdline_checkers,
             'enable_all' in args and args.enable_all)
 
+        # If --stats flag is provided then enable statistics-based checkers
+        # unless explicitly disabled by the user.
+        if 'stats_enabled' in args:
+            # In some Clang versions, statisticsbased checkers are in the alpha
+            # package (alpha.statisticsbased), and maybe not in others. Lets
+            # figure out the actual checker name first.
+            special_return_value = next((
+                checker for checker, _ in checkers
+                if SpecialReturnValueCollector.checker_analyze in checker),
+                None)
+            unchecked_return_value = next((
+                checker for checker, _ in checkers
+                if ReturnValueCollector.checker_analyze in checker),
+                None)
+
+            if special_return_value and not _is_user_disabled_checker(
+                    special_return_value,
+                    cmdline_checkers):
+                handler.set_checker_enabled(special_return_value)
+
+            if unchecked_return_value and not _is_user_disabled_checker(
+                    unchecked_return_value,
+                    cmdline_checkers):
+                handler.set_checker_enabled(unchecked_return_value)
+
+        # Statistics collector checkers must be explicitly disabled as they
+        # trash the output. These checkers are executed in a preceding analysis
+        # phase.
+        handler.set_checker_enabled(
+            SpecialReturnValueCollector.checker_collect, False)
+
+        handler.set_checker_enabled(
+            ReturnValueCollector.checker_collect, False)
+
         handler.checker_config = []
-        r = re.compile(r'(?P<analyzer>.+?):(?P<key>.+?)=(?P<value>.+)')
 
         # TODO: This extra "isinstance" check is needed for
         # CodeChecker checkers --checker-config. This command also runs
         # this function in order to construct a config handler.
         if 'checker_config' in args and isinstance(args.checker_config, list):
             for cfg in args.checker_config:
-                m = re.search(r, cfg)
-                if m.group('analyzer') == cls.ANALYZER_NAME:
+                if cfg.analyzer == cls.ANALYZER_NAME:
                     handler.checker_config.append(
-                        m.group('key') + '=' + m.group('value'))
+                        f"{cfg.checker}:{cfg.option}={cfg.value}")
 
         # TODO: This extra "isinstance" check is needed for
         # CodeChecker analyzers --analyzer-config. This command also runs
@@ -511,9 +705,7 @@ class ClangSA(analyzer_base.SourceAnalyzer):
         if 'analyzer_config' in args and \
                 isinstance(args.analyzer_config, list):
             for cfg in args.analyzer_config:
-                m = re.search(r, cfg)
-                if m.group('analyzer') == cls.ANALYZER_NAME:
-                    handler.checker_config.append(
-                        m.group('key') + '=' + m.group('value'))
+                if cfg.analyzer == cls.ANALYZER_NAME:
+                    handler.checker_config.append(f"{cfg.option}={cfg.value}")
 
         return handler
