@@ -6,6 +6,8 @@
 #
 # -------------------------------------------------------------------------
 """
+Module for handling ClangTidy-related functionality related to analysis,
+checker handling and configuration.
 """
 
 import ast
@@ -14,14 +16,18 @@ import os
 import re
 import shlex
 import subprocess
-from typing import List, Tuple
+from typing import Iterable, List, Set, Tuple
+
+import yaml
 
 from codechecker_common.logger import get_logger
 
-from codechecker_analyzer import env
+from codechecker_analyzer import analyzer_context, env
+from codechecker_analyzer.analyzers.clangsa.analyzer import ClangSA
 
 from .. import analyzer_base
-from ..config_handler import CheckerState, get_compiler_warning_name
+from ..config_handler import CheckerState, CheckerType, \
+    get_compiler_warning_name_and_type
 from ..flag import has_flag
 from ..flag import prepend_all
 
@@ -51,14 +57,51 @@ def parse_checkers(tidy_output):
     return checkers
 
 
+def parse_checker_config_old(config_dump):
+    """
+    Return the parsed clang-tidy config options as a list of
+    (flag, default_value) tuples. This variant works
+    for clang-tidy up to version 14.
+
+    config_dump -- clang-tidy config options YAML dump in pre-LLVM15 format.
+    """
+    reg = re.compile(r'key:\s+(\S+)\s+value:\s+([^\n]+)')
+    all = re.findall(reg, config_dump)
+    # tidy emits the checker option with a "." prefix, but we need a ":"
+    all = [(option[0].replace(".", ":"), option[1]) for option in all]
+    return all
+
+
+def parse_checker_config_new(config_dump):
+    """
+    Return the parsed clang-tidy config options as a list of
+    (flag, default_value) tuples. This variant works
+    for clang-tidy starting with version 15
+
+    config_dump -- clang-tidy config options YAML dump in post-LLVM15 format.
+    """
+    try:
+        data = yaml.safe_load(config_dump)
+        if 'CheckOptions' not in data:
+            return None
+
+        return [[key.replace(".", ":"), value]
+                for (key, value) in data['CheckOptions'].items()]
+    except ImportError:
+        return None
+
+
 def parse_checker_config(config_dump):
     """
     Return the parsed clang-tidy config options as a list of
     (flag, default_value) tuples.
     config_dump -- clang-tidy config options YAML dump.
     """
-    reg = re.compile(r'key:\s+(\S+)\s+value:\s+([^\n]+)')
-    return re.findall(reg, config_dump)
+    result = parse_checker_config_old(config_dump)
+    if not result:
+        result = parse_checker_config_new(config_dump)
+
+    return result
 
 
 def parse_analyzer_config(config_dump):
@@ -70,6 +113,110 @@ def parse_analyzer_config(config_dump):
     return re.findall(r'^(\S+):\s+(\S+)$', config_dump, re.MULTILINE)
 
 
+def get_diagtool_bin():
+    """
+    Return full path of diagtool.
+
+    Select clang binary, check for a 'diagtool' binary next to the selected
+    clang binary and return full path of this binary if it exists.
+    """
+    context = analyzer_context.get_context()
+    clang_bin = context.analyzer_binaries.get(ClangSA.ANALYZER_NAME)
+
+    if not clang_bin:
+        return None
+
+    # Resolve symlink.
+    clang_bin = os.path.realpath(clang_bin)
+
+    # Find diagtool next to the clang binary.
+    diagtool_bin = os.path.join(os.path.dirname(clang_bin), 'diagtool')
+    if os.path.exists(diagtool_bin):
+        return diagtool_bin
+
+    LOG.debug("'diagtool' can not be found next to the clang binary (%s)!",
+              clang_bin)
+
+
+def get_warnings(env=None):
+    """
+    Returns list of warning flags by using diagtool.
+    """
+    diagtool_bin = get_diagtool_bin()
+
+    if not diagtool_bin:
+        return []
+
+    try:
+        result = subprocess.check_output(
+            [diagtool_bin, 'tree'],
+            env=env,
+            universal_newlines=True,
+            encoding="utf-8",
+            errors="ignore")
+        return [w[2:] for w in result.split()
+                if w.startswith("-W") and w != "-W"]
+    except subprocess.CalledProcessError as exc:
+        LOG.error("'diagtool' encountered an error while retrieving the "
+                  "checker list. If you are using a custom compiled clang, "
+                  "you may have forgotten to build the 'diagtool' target "
+                  "alongside 'clang' and 'clang-tidy'! Error message: %s",
+                  exc.output)
+
+        raise
+    except OSError:
+        raise
+
+
+def _add_asterisk_for_group(
+    subset_checkers: Iterable[str],
+    all_checkers: Set[str]
+) -> List[str]:
+    """
+    Since CodeChecker interprets checker name prefixes as checker groups, they
+    have to be added a '*' joker character when using them at clang-tidy
+    -checks flag. This function adds a '*' for each item in "checkers" if it's
+    a checker group, i.e. identified as a prefix for any checker name in
+    "all_checkers".
+    For example "readability-container" is a prefix of multiple checkers, so
+    this is converted to "readability-container-*". On the other hand
+    "performance-trivially-destructible" is a full checker name, so it remains
+    as is.
+    """
+    def is_group_prefix_of(prefix: str, long: str) -> bool:
+        """
+        Returns True if a checker(-group) name is prefix of another
+        checker name. For example bugprone-string is prefix of
+        bugprone-string-constructor but not of
+        bugprone-stringview-nullptr.
+        """
+        prefix_split = prefix.split('-')
+        long_split = long.split('-')
+        return prefix_split == long_split[:len(prefix_split)]
+
+    def need_asterisk(checker: str) -> bool:
+        return any(
+            is_group_prefix_of(checker, long) and checker != long
+            for long in all_checkers)
+
+    result = []
+
+    for checker in subset_checkers:
+        result.append(checker + ('*' if need_asterisk(checker) else ''))
+
+    return result
+
+
+def parse_version(tidy_output):
+    """
+    Parse clang-tidy version output and return the version number.
+    """
+    version_re = re.compile(r'.*version (?P<version>[\d\.]+)', re.S)
+    match = version_re.match(tidy_output)
+    if match:
+        return match.group('version')
+
+
 class ClangTidy(analyzer_base.SourceAnalyzer):
     """
     Constructs the clang tidy analyzer commands.
@@ -77,34 +224,77 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
 
     ANALYZER_NAME = 'clang-tidy'
 
+    # Cache object for get_analyzer_checkers().
+    __analyzer_checkers = None
+
+    @classmethod
+    def analyzer_binary(cls):
+        return analyzer_context.get_context() \
+            .analyzer_binaries[cls.ANALYZER_NAME]
+
+    @classmethod
+    def get_binary_version(self, environ, details=False) -> str:
+        # No need to LOG here, we will emit a warning later anyway.
+        if not self.analyzer_binary():
+            return None
+
+        version = [self.analyzer_binary(), '--version']
+        try:
+            output = subprocess.check_output(version,
+                                             env=environ,
+                                             universal_newlines=True,
+                                             encoding="utf-8",
+                                             errors="ignore")
+            if details:
+                return output.strip()
+            return parse_version(output)
+        except (subprocess.CalledProcessError, OSError) as oerr:
+            LOG.warning("Failed to get analyzer version: %s",
+                        ' '.join(version))
+            LOG.warning(oerr)
+
+        return None
+
     def add_checker_config(self, checker_cfg):
         LOG.error("Not implemented yet")
 
     @classmethod
-    def get_analyzer_checkers(cls, cfg_handler, environ):
+    def get_analyzer_checkers(cls):
         """
         Return the list of the all of the supported checkers.
         """
         try:
+            if cls.__analyzer_checkers:
+                return cls.__analyzer_checkers
+
+            environ = analyzer_context.get_context().analyzer_env
             result = subprocess.check_output(
-                [cfg_handler.analyzer_binary, "-list-checks", "-checks=*"],
+                [cls.analyzer_binary(), "-list-checks", "-checks=*"],
                 env=environ,
                 universal_newlines=True,
                 encoding="utf-8",
                 errors="ignore")
-            return parse_checkers(result)
+            checker_description = parse_checkers(result)
+
+            checker_description.extend(
+                ("clang-diagnostic-" + warning, "")
+                for warning in get_warnings(environ))
+
+            cls.__analyzer_checkers = checker_description
+
+            return checker_description
         except (subprocess.CalledProcessError, OSError):
             return []
 
     @classmethod
-    def get_checker_config(cls, cfg_handler, environ):
+    def get_checker_config(cls):
         """
         Return the checker configuration of the all of the supported checkers.
         """
         try:
             result = subprocess.check_output(
-                [cfg_handler.analyzer_binary, "-dump-config", "-checks=*"],
-                env=environ,
+                [cls.analyzer_binary(), "-dump-config", "-checks=*"],
+                env=analyzer_context.get_context().analyzer_env,
                 universal_newlines=True,
                 encoding="utf-8",
                 errors="ignore")
@@ -113,14 +303,14 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
             return []
 
     @classmethod
-    def get_analyzer_config(cls, cfg_handler, environ):
+    def get_analyzer_config(cls):
         """
         Return the analyzer configuration with all checkers enabled.
         """
         try:
             result = subprocess.check_output(
-                [cfg_handler.analyzer_binary, "-dump-config", "-checks=*"],
-                env=environ,
+                [cls.analyzer_binary(), "-dump-config", "-checks=*"],
+                env=analyzer_context.get_context().analyzer_env,
                 universal_newlines=True,
                 encoding="utf-8",
                 errors="ignore")
@@ -133,8 +323,7 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         Return a list of checkers and warnings what needs to be enabled during
         analysis.
 
-        If the file specified by the '--tidy-config' option contains a 'Checks'
-        key or the 'Checks' option is specified through the '--analyzer-config'
+        If 'Checks' option is specified through '--analyzer-config'
         the return value will be a tuple of empty lists which means do not turn
         checkers explicitly. "clang-analyzer-*" is an exception, because we
         want to disable these even if analyzer config is given. If we wouldn't
@@ -142,42 +331,82 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         clang-tidy and in some cases that would cause analysis error due to
         some ClangSA bug.
         """
-        checkers = []
-        compiler_warnings = []
+        # clang-tidy emits reports from its check in the same format as
+        # compiler diagnostics (like unused variables, etc). This makes it a
+        # little difficult to distinguish compiler warnings and clang-tidy
+        # check warnings. The only clue is that compiler warnings are emitted
+        # as if they came from a check called clang-diagnostic- (e.g.
+        # -Wunused-variable will emit a warning under the name
+        # clang-diagnostic-unused-variable).
+
+        # There are two ways to disable a compiler warning in clang-tidy,
+        # either by -Wno- or -checks=-clang-diagnostic- (note the dash before
+        # clang-diagnostic!). However, there is only one way to enable them:
+        # through -W. Using -checks=clang-diagnostic- does not enable the
+        # warning, but undoes -checks=-clang-diagnostic-.
+
+        # Since we disable all checks by default via -checks=-*, in order to
+        # enable a compiler warning, we first have to undo the -checks level
+        # disable and then enable it, so we need both
+        # -checks=compiler-diagnostic- and -W.
+        compiler_warnings = list()
+        enabled_checkers = list()
 
         has_checker_config = \
             config.checker_config and config.checker_config != '{}'
 
-        # Do not disable any clang-tidy checks explicitly, but don't run
-        # ClangSA checkers. ClangSA checkers are driven by an other
-        # analyzer in CodeChecker.
-        checkers.append('-clang-analyzer-*')
-
-        # For clang compiler warnings a correspoding
-        # clang-diagnostic error is generated by Clang tidy.
-        # They can be disabled by this glob -clang-diagnostic-*
-        checkers.append('clang-diagnostic-*')
-
         # Config handler stores which checkers are enabled or disabled.
         for checker_name, value in config.checks().items():
             state, _ = value
 
-            # Checker name is a compiler warning.
-            warning_name = get_compiler_warning_name(checker_name)
+            warning_name, warning_type = \
+                get_compiler_warning_name_and_type(checker_name)
+
+            # This warning must be given a parameter separated by either '=' or
+            # space. This warning is not supported as a checker name so its
+            # special usage is avoided.
+            if warning_name and warning_name.startswith('frame-larger-than'):
+                continue
+
             if warning_name is not None:
-                if state == CheckerState.enabled:
-                    compiler_warnings.append('-W' + warning_name)
-                elif state == CheckerState.disabled:
-                    compiler_warnings.append('-Wno-' + warning_name)
+                # -W and clang-diagnostic- are added as compiler warnings.
+                if warning_type == CheckerType.compiler:
+                    LOG.warning("As of CodeChecker v6.22, the following usage"
+                                f"of '{checker_name}' compiler warning as a "
+                                "checker name is deprecated, please use "
+                                f"'clang-diagnostic-{checker_name[1:]}' "
+                                "instead.")
+                    if state == CheckerState.enabled:
+                        compiler_warnings.append('-W' + warning_name)
+                        enabled_checkers.append(checker_name)
+                    elif state == CheckerState.disabled:
+                        if config.enable_all:
+                            LOG.warning("Disabling compiler warning with "
+                                        f"compiler flag '-d W{warning_name}' "
+                                        "is not supported.")
+                # If a clang-diagnostic-... is enabled add it as a compiler
+                # warning as -W..., if it is disabled, tidy can suppress when
+                # specified in the -checks parameter list, so we add it there
+                # as -clang-diagnostic-... .
+                elif warning_type == CheckerType.analyzer:
+                    if state == CheckerState.enabled:
+                        compiler_warnings.append('-W' + warning_name)
+                        enabled_checkers.append(checker_name)
+                    else:
+                        compiler_warnings.append('-Wno-' + warning_name)
 
                 continue
-            elif checker_name.startswith('clang-diagnostic'):
-                checker_name += '*'
 
             if state == CheckerState.enabled:
-                checkers.append(checker_name)
-            elif state == CheckerState.disabled:
-                checkers.append('-' + checker_name)
+                enabled_checkers.append(checker_name)
+
+        # By default all checkers are disabled and the enabled ones are added
+        # explicitly.
+        checkers = ['-*']
+
+        checkers += _add_asterisk_for_group(
+            enabled_checkers,
+            set(x[0] for x in ClangTidy.get_analyzer_checkers()))
 
         # -checks=-clang-analyzer-* option is added to the analyzer command by
         # default except when all analyzer config options come from .clang-tidy
@@ -211,7 +440,7 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         try:
             config = self.config_handler
 
-            analyzer_cmd = [config.analyzer_binary]
+            analyzer_cmd = [ClangTidy.analyzer_binary()]
 
             checks, compiler_warnings = self.get_checker_list(config)
 
@@ -235,7 +464,8 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
             analyzer_cmd.append('-Qunused-arguments')
 
             # Enable these compiler warnings by default.
-            analyzer_cmd.extend(['-Wall', '-Wextra'])
+            if not config.enable_all:
+                analyzer_cmd.extend(['-Wno-everything'])
 
             compile_lang = self.buildaction.lang
 
@@ -248,7 +478,7 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
 
             if not has_flag('-arch', analyzer_cmd) and \
                     self.buildaction.arch != "":
-                analyzer_cmd.extend(["-arch ", self.buildaction.arch])
+                analyzer_cmd.extend(["-arch", self.buildaction.arch])
 
             analyzer_cmd.extend(self.buildaction.analyzer_options)
 
@@ -260,7 +490,12 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
                     has_flag('--std', analyzer_cmd):
                 analyzer_cmd.append(self.buildaction.compiler_standard)
 
-            analyzer_cmd.extend(compiler_warnings)
+            if config.enable_all:
+                analyzer_cmd.append("-Weverything")
+                analyzer_cmd.extend(
+                    filter(lambda x: x.startswith('-Wno-'), compiler_warnings))
+            else:
+                analyzer_cmd.extend(compiler_warnings)
 
             return analyzer_cmd
 
@@ -320,14 +555,14 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         return clangtidy
 
     @classmethod
-    def version_compatible(cls, configured_binary, environ):
+    def is_binary_version_incompatible(cls, environ):
         """
-        Check the version compatibility of the given analyzer binary.
+        We support pretty much every Clang-Tidy version.
         """
-        return True
+        return None
 
     def construct_result_handler(self, buildaction, report_output,
-                                 checker_labels, skiplist_handler):
+                                 skiplist_handler):
         """
         See base class for docs.
         """
@@ -335,31 +570,24 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         res_handler = result_handler.ClangTidyResultHandler(
             buildaction, report_output, report_hash)
 
-        res_handler.checker_labels = checker_labels
         res_handler.skiplist_handler = skiplist_handler
         return res_handler
 
     @classmethod
-    def construct_config_handler(cls, args, context):
+    def construct_config_handler(cls, args):
         handler = config_handler.ClangTidyConfigHandler()
-        handler.analyzer_binary = context.analyzer_binaries.get(
-            cls.ANALYZER_NAME)
         handler.report_hash = args.report_hash \
             if 'report_hash' in args else None
 
         # FIXME We cannot get the resource dir from the clang-tidy binary,
         # therefore we get a sibling clang binary which of clang-tidy.
         # TODO Support "clang-tidy -print-resource-dir" .
-        check_env = env.extend(context.path_env_extra,
-                               context.ld_lib_path_extra)
-        # Overwrite PATH to contain only the parent of the clang binary.
-        if os.path.isabs(handler.analyzer_binary):
-            check_env['PATH'] = os.path.dirname(handler.analyzer_binary)
         try:
             with open(args.tidy_args_cfg_file, 'r', encoding='utf-8',
                       errors='ignore') as tidy_cfg:
                 handler.analyzer_extra_arguments = \
-                    re.sub(r'\$\((.*?)\)', env.replace_env_var,
+                    re.sub(r'\$\((.*?)\)',
+                           env.replace_env_var(args.tidy_args_cfg_file),
                            tidy_cfg.read().strip())
                 handler.analyzer_extra_arguments = \
                     shlex.split(handler.analyzer_extra_arguments)
@@ -375,11 +603,14 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         # runs this function in order to construct a config handler.
         if 'analyzer_config' in args and \
                 isinstance(args.analyzer_config, list):
-            r = re.compile(r'(?P<analyzer>.+?):(?P<key>.+?)=(?P<value>.+)')
             for cfg in args.analyzer_config:
-                m = re.search(r, cfg)
-                if m.group('analyzer') == cls.ANALYZER_NAME:
-                    analyzer_config[m.group('key')] = m.group('value')
+                if cfg.analyzer == cls.ANALYZER_NAME:
+                    analyzer_config[cfg.option] = cfg.value
+
+        # Reports in headers are hidden by default in clang-tidy. Re-enable it
+        # if the analyzer doesn't provide any other config options.
+        if not analyzer_config:
+            analyzer_config["HeaderFilterRegex"] = ".*"
 
         # If both --analyzer-config and -config (in --tidyargs) is given then
         # these need to be merged. Since "HeaderFilterRegex" has a default
@@ -406,14 +637,13 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
         # CodeChecker checkers --checker-config. This command also
         # runs this function in order to construct a config handler.
         if 'checker_config' in args and isinstance(args.checker_config, list):
-            r = re.compile(r'(?P<analyzer>.+?):(?P<key>.+?)=(?P<value>.+)')
             check_options = []
 
             for cfg in args.checker_config:
-                m = re.search(r, cfg)
-                if m.group('analyzer') == cls.ANALYZER_NAME:
-                    check_options.append({'key': m.group('key'),
-                                          'value': m.group('value')})
+                if cfg.analyzer == cls.ANALYZER_NAME:
+                    check_options.append({
+                        'key': f"{cfg.checker}.{cfg.option}",
+                        'value': cfg.value})
             analyzer_config['CheckOptions'] = check_options
         else:
             try:
@@ -435,10 +665,7 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
                 analyzer_config.get('take-config-from-directory') != 'true':
             handler.checker_config = json.dumps(analyzer_config)
 
-        check_env = env.extend(context.path_env_extra,
-                               context.ld_lib_path_extra)
-
-        checkers = ClangTidy.get_analyzer_checkers(handler, check_env)
+        checkers = ClangTidy.get_analyzer_checkers()
 
         try:
             cmdline_checkers = args.ordered_checkers
@@ -449,7 +676,6 @@ class ClangTidy(analyzer_base.SourceAnalyzer):
             cmdline_checkers = []
 
         handler.initialize_checkers(
-            context,
             checkers,
             cmdline_checkers,
             'enable_all' in args and args.enable_all)
