@@ -11,6 +11,7 @@ from collections import namedtuple
 # pylint: disable=no-name-in-module
 from distutils.spawn import find_executable
 from enum import Enum
+from pathlib import Path
 
 import glob
 import json
@@ -23,11 +24,10 @@ import tempfile
 import traceback
 from typing import Dict, List, Optional
 
-from codechecker_report_converter.util import load_json_or_empty
-
 from codechecker_analyzer.analyzers import clangsa
 
 from codechecker_common.logger import get_logger
+from codechecker_common.util import load_json
 
 from .. import gcc_toolchain
 from .build_action import BuildAction
@@ -86,18 +86,24 @@ IGNORED_OPTIONS_GCC = [
     '-finline-local-initialisers',
     '-fipa-sra',
     '-fmacro-prefix-map',
+    '-fmerge-constants',
     '-fno-aggressive-loop-optimizations',
+    '-f(no-)?allow-store-data-races',
     '-fno-canonical-system-headers',
+    '-f(no-)?code-hoisting',
     '-fno-delete-null-pointer-checks',
     '-fno-defer-pop',
     '-fno-extended-identifiers',
     '-fno-jump-table',
+    '-fno-keep-inline-dllexport'
     '-fno-keep-static-consts',
+    '-fno-lifetime-dse',
     '-f(no-)?reorder-functions',
     '-fno-strength-reduce',
     '-fno-toplevel-reorder',
     '-fno-unit-at-a-time',
     '-fno-var-tracking-assignments',
+    '-fno-tree-dominator-opts',
     '-fobjc-link-runtime',
     '-fpartial-inlining',
     '-fpeephole2',
@@ -106,6 +112,8 @@ IGNORED_OPTIONS_GCC = [
     '-frename-registers',
     '-frerun-cse-after-loop',
     '-fs$',
+    '-fsanitize=bounds-strict',
+    '-fsched-pressure',
     '-fsched-spec',
     '-fstack-usage',
     '-fstack-reuse',
@@ -159,10 +167,13 @@ IGNORED_OPTIONS_GCC = [
     # --- IGNORED --- #
     '-save-temps',
     # Clang gives different warnings than GCC. Thus if these flags are kept,
-    # '-Werror', '-pedantic-errors' the analysis with Clang can fail even
-    # if the compilation passes with GCC.
+    # '-Werror', '-Wno-error', '-pedantic-errors' the analysis with Clang can
+    # fail even if the compilation passes with GCC.
     '-Werror',
+    '-Wno-error',
     '-pedantic-errors',
+    # Profiling flags are different or behave differently in GCC
+    '-fprofile',
     # Remove the option disabling the warnings.
     '-w',
     '-g(.+)?$',
@@ -215,7 +226,8 @@ COMPILE_OPTIONS = [
     '-Wno-',
     '--sysroot=',
     '-sdkroot',
-    '--gcc-toolchain='
+    '--gcc-toolchain=',
+    '-pthread'
 ]
 
 COMPILE_OPTIONS = re.compile('|'.join(COMPILE_OPTIONS))
@@ -276,7 +288,7 @@ def filter_compiler_includes_extra_args(compiler_flags):
     # If these options are present in the original build command, they must
     # be forwarded to get_compiler_includes and get_compiler_defines so the
     # resulting includes point to the target that was used in the build.
-    pattern = re.compile('-m(32|64)|-std=|-stdlib=|-nostdinc')
+    pattern = re.compile('-m(32|64)|-std=|-stdlib=|-nostdinc|--driver-mode=')
     extra_opts = list(filter(pattern.match, compiler_flags))
 
     pos = next((pos for pos, val in enumerate(compiler_flags)
@@ -542,7 +554,7 @@ class ImplicitCompilerInfo:
         ICI = ImplicitCompilerInfo
         ICI.compiler_info = {}
 
-        contents = load_json_or_empty(file_path, {})
+        contents = load_json(file_path, {})
         for k, v in contents.items():
             k = json.loads(k)
             ICI.compiler_info[
@@ -902,7 +914,6 @@ def parse_options(compilation_db_entry,
                   keep_gcc_include_fixed=False,
                   keep_gcc_intrin=False,
                   get_clangsa_version_func=None,
-                  env=None,
                   analyzer_clang_version=None):
     """
     This function parses a GCC compilation action and returns a BuildAction
@@ -928,7 +939,6 @@ def parse_options(compilation_db_entry,
                             It requires the compiler binary and an env.
                             get_clangsa_version_func(compiler_binary, env)
                             Should return false for a non clang compiler.
-    env -- Is the environment where a subprocess call should be executed.
     analyzer_clang_version -- version information about the clang which is
                               used to execute the analysis
     """
@@ -1003,7 +1013,7 @@ def parse_options(compilation_db_entry,
         # did not find in the cache yet
         try:
             compiler_version_info = \
-                get_clangsa_version_func(details['compiler'], env)
+                get_clangsa_version_func(details['compiler'])
         except (subprocess.CalledProcessError, OSError) as cerr:
             LOG.error('Failed to get and parse version of: %s',
                       details['compiler'])
@@ -1039,6 +1049,11 @@ def parse_options(compilation_db_entry,
         details['action_type'] = BuildAction.COMPILE
 
     details['source'] = compilation_db_entry['file']
+    # Calculate absolute path to file if not already abs
+    if not Path(details['source']).is_absolute():
+        details['source'] = str(Path(
+            Path(compilation_db_entry['directory']).absolute(),
+            compilation_db_entry['file']).resolve())
 
     # In case the file attribute in the entry is empty.
     if details['source'] == '.':
@@ -1070,10 +1085,11 @@ def parse_options(compilation_db_entry,
         gcc_toolchain.toolchain_in_args(details['analyzer_options'])
 
     # Store the compiler built in include paths and defines.
-    # If clang compiler is used for compilation and analysis,
-    # do not collect the implicit include paths.
-    if (not toolchain and not using_same_clang_to_compile_and_analyze) or \
-            (compiler_info_file and os.path.exists(compiler_info_file)):
+    # If clang compiler is used for compilation and analysis, or language is
+    # not recognized, do not collect the implicit include paths.
+    if ((not toolchain and not using_same_clang_to_compile_and_analyze) or
+            (compiler_info_file and os.path.exists(compiler_info_file))) and \
+            details['lang']:
         ImplicitCompilerInfo.set(details, compiler_info_file)
 
     if not keep_gcc_include_fixed:
@@ -1188,6 +1204,8 @@ class CompileActionUniqueingType(Enum):
     # alphabetically first output object file name
     SOURCE_REGEX = 2  # Based on source file, uniqueing by regex filter
     STRICT = 3  # Gives error in case of duplicate
+    SYMLINK = 4  # Based on source file, uniqueing by
+    # recognizing symlink and remove duplication
 
 
 def parse_unique_log(compilation_database,
@@ -1199,7 +1217,6 @@ def parse_unique_log(compilation_database,
                      analysis_skip_handlers=None,
                      pre_analysis_skip_handlers=None,
                      ctu_or_stats_enabled=False,
-                     env=None,
                      analyzer_clang_version=None):
     """
     This function reads up the compilation_database
@@ -1249,7 +1266,6 @@ def parse_unique_log(compilation_database,
                                  skipped during pre analysis
     ctu_or_stats_enabled -- ctu or statistics based analysis was enabled
                             influences the behavior which files are skipped.
-    env -- Is the environment where a subprocess call should be executed.
     analyzer_clang_version -- version information about the clang which is
                               used to execute the analysis
     """
@@ -1262,6 +1278,8 @@ def parse_unique_log(compilation_database,
             build_action_uniqueing = CompileActionUniqueingType.NONE
         elif compile_uniqueing == "strict":
             build_action_uniqueing = CompileActionUniqueingType.STRICT
+        elif compile_uniqueing == "symlink":
+            build_action_uniqueing = CompileActionUniqueingType.SYMLINK
         else:
             build_action_uniqueing = CompileActionUniqueingType.SOURCE_REGEX
             uniqueing_re = re.compile(compile_uniqueing)
@@ -1269,33 +1287,32 @@ def parse_unique_log(compilation_database,
         skipped_cmp_cmd_count = 0
 
         for entry in extend_compilation_database_entries(compilation_database):
-            # Normalization needs to be done here, because the skip regex
-            # won't match properly in the skiplist handler.
-            entry['file'] = os.path.normpath(
-                os.path.join(entry['directory'], entry['file']))
-            # Skip parsing the compilaton commands if it should be skipped
-            # at both analysis phases (pre analysis and analysis).
-            # Skipping of the compile commands is done differently if no
-            # CTU or statistics related feature was enabled.
-            if analysis_skip_handlers \
-                and analysis_skip_handlers.should_skip(entry['file']) \
-                and (not ctu_or_stats_enabled or pre_analysis_skip_handlers
-                     and pre_analysis_skip_handlers.should_skip(
-                         entry['file'])):
-                skipped_cmp_cmd_count += 1
-                continue
-
+            # Parse compilation db entry into an action object
             action = parse_options(entry,
                                    compiler_info_file,
                                    keep_gcc_include_fixed,
                                    keep_gcc_intrin,
                                    clangsa.version.get,
-                                   env,
                                    analyzer_clang_version)
 
+            # Skip parsing the compilaton commands if it should be skipped
+            # at both analysis phases (pre analysis and analysis).
+            # Skipping of the compile commands is done differently if no
+            # CTU or statistics related feature was enabled.
+            if analysis_skip_handlers \
+                and analysis_skip_handlers.should_skip(action.source) \
+                and (not ctu_or_stats_enabled or pre_analysis_skip_handlers
+                     and pre_analysis_skip_handlers.should_skip(
+                         action.source)):
+                skipped_cmp_cmd_count += 1
+                LOG.debug("skipping:", action.source)
+                continue
+
             if not action.lang:
+                skipped_cmp_cmd_count += 1
                 continue
             if action.action_type != BuildAction.COMPILE:
+                skipped_cmp_cmd_count += 1
                 continue
             if build_action_uniqueing == CompileActionUniqueingType.NONE:
                 if action not in uniqued_build_actions:
@@ -1317,6 +1334,10 @@ def parse_unique_log(compilation_database,
                 elif action.output <\
                         uniqued_build_actions[action.source].output:
                     uniqued_build_actions[action.source] = action
+            elif build_action_uniqueing == CompileActionUniqueingType.SYMLINK:
+                real_path = os.path.realpath(entry['file'])
+                if real_path not in uniqued_build_actions:
+                    uniqued_build_actions[real_path] = action
             elif build_action_uniqueing ==\
                     CompileActionUniqueingType.SOURCE_REGEX:
                 LOG.debug("uniqueing regex")
