@@ -9,32 +9,31 @@
 Execute analysis over an already existing build.json compilation database.
 """
 
-
 import argparse
 import collections
 import json
-import multiprocessing
 import os
-import re
-import shlex
 import shutil
 import sys
-
 from typing import List
+from pathlib import Path
 
-from codechecker_report_converter.util import load_json_or_empty
 from tu_collector import tu_collector
 
-from codechecker_analyzer import analyzer, analyzer_context, env
+from codechecker_analyzer import analyzer, analyzer_context, \
+    compilation_database
 from codechecker_analyzer.analyzers import analyzer_types, clangsa
 from codechecker_analyzer.arg import \
-        OrderedCheckersAction, OrderedConfigAction
+    OrderedCheckersAction, OrderedConfigAction, existing_abspath, \
+    analyzer_config, checker_config, AnalyzerConfig, CheckerConfig
+
 from codechecker_analyzer.buildlog import log_parser
 
-from codechecker_common import arg, logger, cmd_config
+from codechecker_common import arg, logger, cmd_config, review_status_handler
+from codechecker_common.compatibility.multiprocessing import cpu_count
 from codechecker_common.skiplist_handler import SkipListHandler, \
     SkipListHandlers
-
+from codechecker_common.util import load_json
 
 LOG = logger.get_logger('system')
 
@@ -44,6 +43,11 @@ header_file_extensions = (
 epilog_env_var = f"""
   CC_ANALYZERS_FROM_PATH   Set to `yes` or `1` to enforce taking the analyzers
                            from the `PATH` instead of the given binaries.
+  CC_ANALYZER_BIN          Set the absolute paths of an analyzer binaries.
+                           Overrides other means of CodeChecker getting hold of
+                           binary.
+                           Format: CC_ANALYZER_BIN='<analyzer1>:/path/to/bin1;
+                                                    <analyzer2>:/path/to/bin2'
   CC_CLANGSA_PLUGIN_DIR    If the CC_ANALYZERS_FROM_PATH environment variable
                            is set you can configure the plugin directory of the
                            Clang Static Analyzer by using this environment
@@ -153,18 +157,18 @@ def add_arguments_to_parser(parser):
     Add the subcommand's arguments to the given argparse.ArgumentParser.
     """
 
-    parser.add_argument('logfile',
-                        type=str,
-                        help="Path to the JSON compilation command database "
-                             "files which were created during the build. "
-                             "The analyzers will check only the files "
-                             "registered in these build databases.")
+    parser.add_argument('input',
+                        type=existing_abspath,
+                        help="The input of the analysis can be either a "
+                             "compilation database JSON file, a path to a "
+                             "source file or a path to a directory containing "
+                             "source files.")
 
     parser.add_argument('-j', '--jobs',
                         type=int,
                         dest="jobs",
                         required=False,
-                        default=multiprocessing.cpu_count(),
+                        default=cpu_count(),
                         help="Number of threads to use in analysis. More "
                              "threads mean faster analysis at the cost of "
                              "using more memory.")
@@ -193,6 +197,15 @@ def add_arguments_to_parser(parser):
                                 "Example: '/path/to/main.cpp', 'lib/*.cpp', "
                                 "*/test*'.")
 
+    parser.add_argument('--review-status-config',
+                        dest="review_status_config",
+                        required=False,
+                        type=existing_abspath,
+                        default=argparse.SUPPRESS,
+                        help="Path of review_status.yaml config file which "
+                             "contains review status settings assigned to "
+                             "specific directories, checkers, bug hashes.")
+
     parser.add_argument('-o', '--output',
                         dest="output_path",
                         required=True,
@@ -287,6 +300,10 @@ def add_arguments_to_parser(parser):
                              "and an error is given if "
                              "there is more than one compilation "
                              "action for a source file. "
+                             "symlink: recognizes symlinks and removes "
+                             "duplication in the compilation database to "
+                             "ensure that each source file is "
+                             "analyzed only once. "
                              "alpha(default in CTU mode): If there is more "
                              "than one compilation action for a source "
                              "file, only the one is kept that belongs to the "
@@ -375,6 +392,13 @@ def add_arguments_to_parser(parser):
 
     cmd_config.add_option(analyzer_opts)
 
+    analyzer_opts.add_argument('--cppcheckargs',
+                               dest="cppcheck_args_cfg_file",
+                               required=False,
+                               default=argparse.SUPPRESS,
+                               help="File containing argument which will be "
+                                    "forwarded verbatim for Cppcheck.")
+
     analyzer_opts.add_argument('--saargs',
                                dest="clangsa_args_cfg_file",
                                required=False,
@@ -394,17 +418,19 @@ def add_arguments_to_parser(parser):
                                dest='tidy_config',
                                required=False,
                                default=argparse.SUPPRESS,
-                               help="A file in YAML format containing the "
+                               help="DEPRECATED. "
+                                    "A file in YAML format containing the "
                                     "configuration of clang-tidy checkers. "
                                     "The file can be dumped by "
                                     "'CodeChecker analyzers --dump-config "
                                     "clang-tidy' command.")
 
     analyzer_opts.add_argument('--analyzer-config',
+                               type=analyzer_config,
                                dest='analyzer_config',
                                nargs='*',
                                action=OrderedConfigAction,
-                               default=["clang-tidy:HeaderFilterRegex=.*"],
+                               default=argparse.SUPPRESS,
                                help="Analyzer configuration options in the "
                                     "following format: analyzer:key=value. "
                                     "The collection of the options can be "
@@ -413,9 +439,8 @@ def add_arguments_to_parser(parser):
                                     "--analyzer-config'.\n"
                                     "If the file at --tidyargs "
                                     "contains a -config flag then those "
-                                    "options extend these and override "
-                                    "\"HeaderFilterRegex\" if any.\n"
-                                    "To use analyzer configuration file "
+                                    "options extend these.\n"
+                                    "To use an analyzer configuration file "
                                     "in case of Clang Tidy (.clang-tidy) use "
                                     "the "
                                     "'clang-tidy:take-config-from-directory="
@@ -424,6 +449,7 @@ def add_arguments_to_parser(parser):
                                     "binary.")
 
     analyzer_opts.add_argument('--checker-config',
+                               type=checker_config,
                                dest='checker_config',
                                nargs='*',
                                action=OrderedConfigAction,
@@ -447,8 +473,7 @@ def add_arguments_to_parser(parser):
                                     "the analysis is considered as a failed "
                                     "one.")
 
-    context = analyzer_context.get_context()
-    clang_has_z3 = analyzer_types.is_z3_capable(context)
+    clang_has_z3 = analyzer_types.is_z3_capable()
 
     if clang_has_z3:
         analyzer_opts.add_argument('--z3',
@@ -464,7 +489,7 @@ def add_arguments_to_parser(parser):
                                         "backend is a highly experimental "
                                         "and likely unstable feature.")
 
-    clang_has_z3_refutation = analyzer_types.is_z3_refutation_capable(context)
+    clang_has_z3_refutation = analyzer_types.is_z3_refutation_capable()
 
     if clang_has_z3_refutation:
         analyzer_opts.add_argument('--z3-refutation',
@@ -482,7 +507,7 @@ def add_arguments_to_parser(parser):
                                         "that much of a slowdown compared to "
                                         "using only the Z3 solver.")
 
-    if analyzer_types.is_ctu_capable(context):
+    if analyzer_types.is_ctu_capable():
         ctu_opts = parser.add_argument_group(
             "cross translation unit analysis arguments",
             """
@@ -530,14 +555,13 @@ Cross-TU analysis. By default, no CTU analysis is run when
                               action='store_true',
                               dest='ctu_reanalyze_on_failure',
                               default=argparse.SUPPRESS,
-                              help="DEPRECATED. The flag will be removed. "
-                                   "If Cross-TU analysis is enabled and fails "
+                              help="If Cross-TU analysis is enabled and fails "
                                    "for some reason, try to re analyze the "
                                    "same translation unit without "
                                    "Cross-TU enabled.")
 
         # Only check for AST loading modes if CTU is available.
-        if analyzer_types.is_ctu_on_demand_available(context):
+        if analyzer_types.is_ctu_on_demand_available():
             ctu_opts.add_argument('--ctu-ast-mode',
                                   action='store',
                                   dest='ctu_ast_mode',
@@ -558,7 +582,7 @@ Cross-TU analysis. By default, no CTU analysis is run when
                                        "phase of the analysis. (default: "
                                        "parse-on-demand)")
 
-    if analyzer_types.is_statistics_capable(context):
+    if analyzer_types.is_statistics_capable():
         stat_opts = parser.add_argument_group(
             "Statistics analysis feature arguments",
             """
@@ -628,30 +652,27 @@ statisticsCollector.SpecialReturnValue checkers are available).""")
         """
 Checkers
 ------------------------------------------------
-The analyzer performs checks that are categorized into families or "checkers".
-See 'CodeChecker checkers' for the list of available checkers. You can
-fine-tune which checkers to use in the analysis by setting the enabled and
-disabled flags starting from the bigger groups and going inwards, e.g.
-'-e core -d core.uninitialized -e core.uninitialized.Assign' will enable every
-'core' checker, but only 'core.uninitialized.Assign' from the
-'core.uninitialized' group. Please consult the manual for details. Disabling
-certain checkers - such as the 'core' group - is unsupported by the LLVM/Clang
-community, and thus discouraged.
+An analyzer checks the source code with the help of checkers. Checkers
+implement a specific rule, such as "don't divide by zero", and emit a warning
+if the corresponding rule is violated. Available checkers can be listed by
+'CodeChecker checkers'.
+
+Checkers are grouped by CodeChecker via labels (described below), and sometimes
+by their analyzer tool. An example for the latter is 'clangsa', which orders
+checkers in a package hierarchy (e.g. in 'core.uninitialized.Assign', 'core'
+and 'core.uninitialized' are packages).
 
 Compiler warnings and errors
 ------------------------------------------------
 Compiler warnings are diagnostic messages that report constructions that are
 not inherently erroneous but that are risky or suggest there may have been an
-error. Compiler warnings are named 'clang-diagnostic-<warning-option>', e.g.
-Clang warning controlled by '-Wliteral-conversion' will be reported with check
-name 'clang-diagnostic-literal-conversion'. You can fine-tune which warnings to
-use in the analysis by setting the enabled and disabled flags starting from the
-bigger groups and going inwards, e.g. '-e Wunused -d Wno-unused-parameter' will
-enable every 'unused' warnings except 'unused-parameter'. These flags should
-start with a capital 'W' or 'Wno-' prefix followed by the waning name (E.g.:
-'-e Wliteral-conversion', '-d Wno-literal-conversion'). By default '-Wall' and
-'-Wextra' warnings are enabled. For more information see:
-https://clang.llvm.org/docs/DiagnosticsReference.html.
+error. However, CodeChecker views them as regular checkers.
+
+Compiler warning names are transformed by CodeChecker to reflect the analyzer
+name. For example, '-Wliteral-conversion' from clang-tidy is transformed to
+'clang-diagnostic-literal-conversion'. However, they need to be enabled by
+their original name, e.g. '-e Wliteral-conversion'.
+
 Sometimes GCC is more permissive than Clang, so it is possible that a specific
 construction doesn't compile with Clang but compiles with GCC. These
 compiler errors are also collected as CodeChecker reports as
@@ -661,20 +682,38 @@ was emitted by clang-tidy.
 
 Checker labels
 ------------------------------------------------
-In CodeChecker there is a manual grouping of checkers. These groups are
-determined by labels. The collection of labels is found in
-config/labels directory. The goal of these labels is that you can
-enable or disable checkers by these labels. See the --label flag of
-"CodeChecker checkers" command.
+Each checker is assigned several '<label>:<value>' pairs. For instance,
+'cppcheck-deallocret' has the labels 'profile:default' and 'severity:HIGH'. The
+goal of labels is that you can enable or disable a batch of checkers with them.
+
+You can enable/disable checkers belonging to a label: '-e <label>:<value>',
+e.g. '-e profile:default'.
+
+See "CodeChecker checkers --help" to learn more.
 
 Guidelines
 ------------------------------------------------
-There are several coding guidelines like CppCoreGuideline, SEI-CERT, etc. These
-are collections of best programming practices to avoid common programming
-errors. Some checkers cover the rules of these guidelines. In CodeChecker there
-is a mapping between guidelines and checkers. This way you can list and enable
+CodeChecker recognizes several third party coding guidelines, such as
+CppCoreGuidelines, SEI-CERT, or MISRA. These are collections of best
+programming practices to avoid common programming errors. Some checkers cover
+the rules of these guidelines. CodeChecker assigns the 'guideline' label to
+these checkers, such as 'guideline:sei-cert'. This way you can list and enable
 those checkers which check the fulfillment of certain guideline rules. See the
-output of "CodeChecker checkers --guideline" command.""")
+output of "CodeChecker checkers --guideline" command.
+
+Guidelines are labels themselves, and can be used as a label:
+'-e guideline:<value>', e.g. '-e guideline:sei-cert'.
+
+Batch enabling/disabling checkers
+------------------------------------------------
+You can fine-tune which checkers to use in the analysis by setting the enable
+and disable flags starting from the bigger groups and going inwards. Taking
+for example the package hierarchy of 'clangsa', '-e core -d core.uninitialized
+-e core.uninitialized.Assign' will enable every 'core' checker, but only
+'core.uninitialized.Assign' from the 'core.uninitialized' group. Mind that
+disabling certain checkers - such as the 'core' group is unsupported by the
+LLVM/Clang community, and thus discouraged.
+""")
 
     checkers_opts.add_argument('-e', '--enable',
                                dest="enable",
@@ -733,11 +772,116 @@ output of "CodeChecker checkers --guideline" command.""")
                                     "the analysis. USE WISELY AND AT YOUR "
                                     "OWN RISK!")
 
+    checkers_opts.add_argument('--disable-all',
+                               dest="disable_all",
+                               required=False,
+                               default=argparse.SUPPRESS,
+                               action='store_true',
+                               help="Disable all checkers of all analyzers. "
+                                    "It is equivalent to using \"--disable "
+                                    "default\" as the first argument.")
+
+    checkers_opts.add_argument('--no-missing-checker-error',
+                               dest="no_missing_checker_error",
+                               action='store_true',
+                               required=False,
+                               default=argparse.SUPPRESS,
+                               help="Emit a warning instead of an error when "
+                                    "an unknown checker name is given to "
+                                    "either --enable, --disable,"
+                                    "--analyzer-config and/or "
+                                    "--checker-config.")
+
     logger.add_verbose_arguments(parser)
     parser.set_defaults(
         func=main, func_process_config_file=cmd_config.process_config_file)
 
 
+def is_analyzer_config_valid(analyzer_conf: List[AnalyzerConfig]) -> bool:
+    """
+    Ensure that the analyzer_config parameter is set to a valid value
+    by verifying if it belongs to the set of allowed values.
+    """
+    wrong_configs = []
+    supported_analyzers = analyzer_types.supported_analyzers
+
+    for cfg in analyzer_conf:
+        if cfg.analyzer not in supported_analyzers:
+            wrong_configs.append((cfg.analyzer, None))
+            continue
+
+        analyzer_class = supported_analyzers[cfg.analyzer]
+        analyzer_name = analyzer_class.ANALYZER_NAME
+        analyzers = [analyzer[0] for analyzer in
+                     analyzer_class.get_analyzer_config()]
+
+        # Make sure "[clang-tidy] Allow to override checker list #3203" works
+        if analyzer_name == 'clang-tidy':
+            analyzers.append('take-config-from-directory')
+
+        if cfg.analyzer == analyzer_name and cfg.option not in analyzers:
+            wrong_configs.append((analyzer_name, cfg.option))
+
+    if wrong_configs:
+        for analyzer_name, option in wrong_configs:
+            if option is None:
+                LOG.error(
+                    f"Invalid argument to --analyzer-config: {analyzer_name} "
+                    f"is not a supported analyzer. Supported analyzers are: "
+                    f"{', '.join(a for a in supported_analyzers)}.")
+            else:
+                LOG.error(
+                    f"Invalid argument to --analyzer-config: {analyzer_name} "
+                    f"has no config named {option}. Use the 'CodeChecker "
+                    f"analyzers --analyzer-config {analyzer_name}' command to "
+                    f"see available configs.")
+
+        return False
+
+    return True
+
+
+def is_checker_config_valid(checker_conf: List[CheckerConfig]) -> bool:
+    """
+    Ensure that the checker_config parameter is set to a valid value
+    by verifying if it belongs to the set of allowed values.
+    """
+    wrong_configs = []
+    supported_analyzers = analyzer_types.supported_analyzers
+
+    for cfg in checker_conf:
+        if cfg.analyzer not in supported_analyzers:
+            wrong_configs.append((cfg.analyzer, None))
+            continue
+
+        analyzer_class = supported_analyzers[cfg.analyzer]
+        analyzer_name = analyzer_class.ANALYZER_NAME
+        checker_conf_opt = f"{cfg.checker}:{cfg.option}"
+        checkers = [checker[0] for checker in
+                    analyzer_class.get_checker_config()]
+
+        if cfg.analyzer == analyzer_name and checker_conf_opt not in checkers:
+            wrong_configs.append((analyzer_name, checker_conf_opt))
+
+    if wrong_configs:
+        for analyzer_name, conf_opt in wrong_configs:
+            if conf_opt is None:
+                LOG.error(
+                    f"Invalid argument to --checker-config: {analyzer_name} "
+                    f"is not a supported analyzer. Supported analyzers are: "
+                    f"{', '.join(a for a in supported_analyzers)}.")
+            else:
+                LOG.error(
+                    f"Invalid argument to --checker-config: invalid checker "
+                    f"and/or checker option '{conf_opt}' for {analyzer_name}. "
+                    f"Use the 'CodeChecker checkers --checker-config' command "
+                    f"to see available checkers.")
+
+        return False
+
+    return True
+
+
 def get_affected_file_paths(
     file_filters: List[str],
     compile_commands: tu_collector.CompilationDB
@@ -748,7 +892,8 @@ def get_affected_file_paths(
     """
     file_paths = []  # Use list to keep the order of the file paths.
     for file_filter in file_filters:
-        file_paths.append(file_filter)
+        file_paths.append(str(Path(file_filter).resolve())
+                          if '*' not in file_filter else file_filter)
 
         if os.path.exists(file_filter) and \
                 file_filter.endswith(header_file_extensions):
@@ -809,6 +954,18 @@ def __update_skip_file(args):
         shutil.copyfile(args.skipfile, skip_file_to_send)
 
 
+def __update_review_status_config(args):
+    rs_config_to_send = os.path.join(args.output_path, 'review_status.yaml')
+
+    if os.path.exists(rs_config_to_send):
+        os.remove(rs_config_to_send)
+
+    if 'review_status_config' in args:
+        LOG.debug("Copying review status config file %s to %s",
+                  args.review_status_config, rs_config_to_send)
+        os.symlink(args.review_status_config, rs_config_to_send)
+
+
 def __cleanup_metadata(metadata_prev, metadata):
     """ Cleanup metadata.
 
@@ -855,29 +1012,36 @@ def __get_result_source_files(metadata):
     return result_src_files
 
 
-def __change_args_to_command_in_comp_db(compile_commands):
-    """
-    In CodeChecker we support compilation databases where the JSON object of a
-    build action contains "file", "directory" and "command" fields. However,
-    compilation databases from intercept build contain "arguments" instead of
-    "command" which is a list of command-line arguments instead of the same
-    command as a single string. This function make this appropriate conversion.
-    """
-    for cc in compile_commands:
-        if 'command' not in cc:
-            # TODO: shlex.join(cmd) would be more elegant after upgrading to
-            # Python 3.8.
-            cc['command'] = ' '.join(map(shlex.quote, cc['arguments']))
-            del cc['arguments']
-
-
 def main(args):
     """
-    Perform analysis on the given logfiles and store the results in a machine-
-    readable format.
+    Perform analysis on the given inputs. Possible inputs are a compilation
+    database, a source file or the path of a project. The analysis results are
+    stored to a report directory given by -o flag.
     """
     logger.setup_logger(args.verbose if 'verbose' in args else None)
 
+    # Validate analyzer and checker config (if any)
+    config_validator = {
+        'analyzer_config': is_analyzer_config_valid,
+        'checker_config': is_checker_config_valid
+    }
+
+    config_validator_res = [validate_func(getattr(args, conf))
+                            for conf, validate_func in config_validator.items()
+                            if conf in args]
+
+    if False in config_validator_res \
+            and 'no_missing_checker_error' not in args:
+        LOG.info("Although it is not recommended, if you want to suppress "
+                 "errors relating to unknown analyzer/checker configs, "
+                 "consider using the option '--no-missing-checker-error'")
+        sys.exit(1)
+
+    if 'tidy_config' in args:
+        LOG.warning(
+            "--tidy-config is deprecated and will be removed in the next "
+            "release. Use --analyzer-config or --checker-config instead.")
+
     # CTU loading mode is only meaningful if CTU itself is enabled.
     if 'ctu_ast_mode' in args and 'ctu_phases' not in args:
         LOG.error("Analyzer option 'ctu-ast-mode' requires CTU mode enabled")
@@ -889,10 +1053,6 @@ def main(args):
         LOG.error(fnerr)
         sys.exit(1)
 
-    if not os.path.exists(args.logfile):
-        LOG.error("The specified logfile '%s' does not exist!", args.logfile)
-        sys.exit(1)
-
     args.output_path = os.path.abspath(args.output_path)
     if os.path.exists(args.output_path) and \
             not os.path.isdir(args.output_path):
@@ -902,23 +1062,10 @@ def main(args):
 
     if 'enable_all' in args:
         LOG.info("'--enable-all' was supplied for this analysis.")
-
-    config_option_re = re.compile(r'^({}):.+=.+$'.format(
-        '|'.join(analyzer_types.supported_analyzers)))
-
-    # Check the format of analyzer options.
-    if 'analyzer_config' in args:
-        for config in args.analyzer_config:
-            if not re.match(config_option_re, config):
-                LOG.error("Analyzer option in wrong format: %s", config)
-                sys.exit(1)
-
-    # Check the format of checker options.
-    if 'checker_config' in args:
-        for config in args.checker_config:
-            if not re.match(config_option_re, config):
-                LOG.error("Checker option in wrong format: %s", config)
-                sys.exit(1)
+    if 'disable_all' in args:
+        if 'ordered_checkers' not in args:
+            args.ordered_checkers = []
+        args.ordered_checkers.insert(0, ('default', False))
 
     # Enable alpha uniqueing by default if ctu analysis is used.
     if 'none' in args.compile_uniqueing and 'ctu_phases' in args:
@@ -933,13 +1080,22 @@ def main(args):
             sys.exit(1)
         compiler_info_file = args.compiler_info_file
 
-    compile_commands = load_json_or_empty(args.logfile)
+    compile_commands = \
+        compilation_database.gather_compilation_database(args.input)
     if compile_commands is None:
+        LOG.error(f"Found no compilation commands in '{args.input}'")
         sys.exit(1)
-    __change_args_to_command_in_comp_db(compile_commands)
 
     # Process the skip list if present.
     skip_handlers = __get_skip_handlers(args, compile_commands)
+    rs_handler = review_status_handler.ReviewStatusHandler(args.output_path)
+
+    try:
+        if 'review_status_config' in args:
+            rs_handler.set_review_status_config(args.review_status_config)
+    except ValueError as ex:
+        LOG.error(ex)
+        sys.exit(1)
 
     ctu_or_stats_enabled = False
     # Skip list is applied only in pre-analysis
@@ -964,8 +1120,6 @@ def main(args):
         ctu_or_stats_enabled = True
 
     context = analyzer_context.get_context()
-    analyzer_env = env.extend(context.path_env_extra,
-                              context.ld_lib_path_extra)
 
     # Number of all the compilation commands in the parsed log files,
     # logged by the logger.
@@ -1001,8 +1155,7 @@ def main(args):
 
     analyzer_clang_version = None
     if analyzer_clang_binary:
-        analyzer_clang_version = clangsa.version.get(analyzer_clang_binary,
-                                                     analyzer_env)
+        analyzer_clang_version = clangsa.version.get(analyzer_clang_binary)
 
     actions, skipped_cmp_cmd_count = log_parser.parse_unique_log(
         compile_commands,
@@ -1014,13 +1167,12 @@ def main(args):
         skip_handlers,
         pre_analysis_skip_handlers,
         ctu_or_stats_enabled,
-        analyzer_env,
         analyzer_clang_version)
 
     if not actions:
-        LOG.info("No analysis is required.\nThere were no compilation "
-                 "commands in the provided compilation database or "
-                 "all of them were skipped.")
+        LOG.warning("No analysis is required.")
+        LOG.warning("There were no compilation commands in the provided "
+                    "compilation database or all of them were skipped.")
         sys.exit(0)
 
     uniqued_compilation_db_file = os.path.join(
@@ -1052,7 +1204,7 @@ def main(args):
     metadata_file = os.path.join(args.output_path, 'metadata.json')
     metadata_prev = None
     if os.path.exists(metadata_file):
-        metadata_prev = load_json_or_empty(metadata_file)
+        metadata_prev = load_json(metadata_file)
         metadata_tool['result_source_files'] = \
             __get_result_source_files(metadata_prev)
 
@@ -1081,11 +1233,11 @@ def main(args):
     LOG.debug_analyzer("Compile commands forwarded for analysis: %d",
                        compile_cmd_count.analyze)
 
-    analyzer.perform_analysis(args, skip_handlers, context, actions,
-                              metadata_tool,
-                              compile_cmd_count)
+    analyzer.perform_analysis(args, skip_handlers, rs_handler, actions,
+                              metadata_tool, compile_cmd_count)
 
     __update_skip_file(args)
+    __update_review_status_config(args)
 
     LOG.debug("Cleanup metadata file started.")
     __cleanup_metadata(metadata_prev, metadata)
@@ -1098,16 +1250,8 @@ def main(args):
 
     # WARN: store command will search for this file!!!!
     compile_cmd_json = os.path.join(args.output_path, 'compile_cmd.json')
-    try:
-        source = os.path.abspath(args.logfile)
-        target = os.path.abspath(compile_cmd_json)
-
-        if source != target:
-            shutil.copyfile(source, target)
-    except shutil.Error:
-        LOG.debug("Compilation database JSON file is the same.")
-    except Exception:
-        LOG.debug("Copying compilation database JSON file failed.")
+    with open(compile_cmd_json, 'w', encoding="utf-8", errors="ignore") as f:
+        json.dump(compile_commands, f, indent=2)
 
     try:
         # pylint: disable=no-name-in-module
