@@ -12,19 +12,19 @@ stdout.
 """
 
 import argparse
-import multiprocessing
 import os
 import shutil
 import sys
 import tempfile
 
-from codechecker_analyzer import analyzer_context
 from codechecker_analyzer.analyzers import analyzer_types
 from codechecker_analyzer.arg import \
-        OrderedCheckersAction, OrderedConfigAction
+    OrderedCheckersAction, OrderedConfigAction, \
+    analyzer_config, checker_config, existing_abspath
 
 from codechecker_common import arg, cmd_config, logger
-from codechecker_report_converter.source_code_comment_handler import \
+from codechecker_common.compatibility.multiprocessing import cpu_count
+from codechecker_common.source_code_comment_handler import \
     REVIEW_STATUS_VALUES
 
 from codechecker_analyzer.cmd.analyze import \
@@ -182,7 +182,7 @@ used to generate a log file on the fly.""")
                                type=int,
                                dest="jobs",
                                required=False,
-                               default=multiprocessing.cpu_count(),
+                               default=cpu_count(),
                                help="Number of threads to use in analysis. "
                                     "More threads mean faster analysis at "
                                     "the cost of using more memory.")
@@ -215,6 +215,10 @@ used to generate a log file on the fly.""")
                              "and an error is given if "
                              "there is more than one compilation "
                              "action for a source file. "
+                             "symlink: recognizes symlinks and removes "
+                             "duplication in the compilation database to "
+                             "ensure that each source file is "
+                             "analyzed only once. "
                              "alpha(default in CTU mode): If there is more "
                              "than one compilation action for a source "
                              "file, only the one is kept that belongs to the "
@@ -321,6 +325,13 @@ used to generate a log file on the fly.""")
     cmd_config.add_option(analyzer_opts)
 
     # TODO: One day, get rid of these. See Issue #36, #427.
+    analyzer_opts.add_argument('--cppcheckargs',
+                               dest="cppcheck_args_cfg_file",
+                               required=False,
+                               default=argparse.SUPPRESS,
+                               help="File containing argument which will be "
+                                    "forwarded verbatim for Cppcheck.")
+
     analyzer_opts.add_argument('--saargs',
                                dest="clangsa_args_cfg_file",
                                required=False,
@@ -341,17 +352,19 @@ used to generate a log file on the fly.""")
                                dest='tidy_config',
                                required=False,
                                default=argparse.SUPPRESS,
-                               help="A file in YAML format containing the "
+                               help="DEPRECATED. "
+                                    "A file in YAML format containing the "
                                     "configuration of clang-tidy checkers. "
                                     "The file can be dumped by "
                                     "'CodeChecker analyzers --dump-config "
                                     "clang-tidy' command.")
 
     analyzer_opts.add_argument('--analyzer-config',
+                               type=analyzer_config,
                                dest='analyzer_config',
                                nargs='*',
                                action=OrderedConfigAction,
-                               default=["clang-tidy:HeaderFilterRegex=.*"],
+                               default=argparse.SUPPRESS,
                                help="Analyzer configuration options in the "
                                     "following format: analyzer:key=value. "
                                     "The collection of the options can be "
@@ -360,9 +373,8 @@ used to generate a log file on the fly.""")
                                     "--analyzer-config'.\n"
                                     "If the file at --tidyargs "
                                     "contains a -config flag then those "
-                                    "options extend these and override "
-                                    "\"HeaderFilterRegex\" if any.\n"
-                                    "To use analyzer configuration file "
+                                    "options extend these.\n"
+                                    "To use an analyzer configuration file "
                                     "in case of Clang Tidy (.clang-tidy) use "
                                     "the "
                                     "'clang-tidy:take-config-from-directory="
@@ -371,6 +383,7 @@ used to generate a log file on the fly.""")
                                     "binary.")
 
     analyzer_opts.add_argument('--checker-config',
+                               type=checker_config,
                                dest='checker_config',
                                nargs='*',
                                action=OrderedConfigAction,
@@ -394,8 +407,17 @@ used to generate a log file on the fly.""")
                                     "the analysis is considered as a failed "
                                     "one.")
 
-    context = analyzer_context.get_context()
-    clang_has_z3 = analyzer_types.is_z3_capable(context)
+    analyzer_opts.add_argument('--review-status-config',
+                               dest="review_status_config",
+                               required=False,
+                               type=existing_abspath,
+                               default=argparse.SUPPRESS,
+                               help="Path of review_status.yaml config file "
+                                    "which contains review status settings "
+                                    "assigned to specific directories, "
+                                    "checkers, bug hashes.")
+
+    clang_has_z3 = analyzer_types.is_z3_capable()
 
     if clang_has_z3:
         analyzer_opts.add_argument('--z3',
@@ -411,7 +433,7 @@ used to generate a log file on the fly.""")
                                         "backend is a highly experimental "
                                         "and likely unstable feature.")
 
-    clang_has_z3_refutation = analyzer_types.is_z3_refutation_capable(context)
+    clang_has_z3_refutation = analyzer_types.is_z3_refutation_capable()
 
     if clang_has_z3_refutation:
         analyzer_opts.add_argument('--z3-refutation',
@@ -429,7 +451,7 @@ used to generate a log file on the fly.""")
                                         "that much of a slowdown compared to "
                                         "using only the Z3 solver.")
 
-    if analyzer_types.is_ctu_capable(context):
+    if analyzer_types.is_ctu_capable():
         ctu_opts = parser.add_argument_group(
             "cross translation unit analysis arguments",
             """
@@ -478,14 +500,13 @@ is called.""")
                               action='store_true',
                               dest='ctu_reanalyze_on_failure',
                               default=argparse.SUPPRESS,
-                              help="DEPRECATED. The flag will be removed. "
-                                   "If Cross-TU analysis is enabled and "
+                              help="If Cross-TU analysis is enabled and "
                                    "fails for some reason, try to re analyze "
                                    "the same translation unit without "
                                    "Cross-TU enabled.")
 
         # Only check for AST loading modes if CTU is available.
-        if analyzer_types.is_ctu_on_demand_available(context):
+        if analyzer_types.is_ctu_on_demand_available():
             ctu_opts.add_argument('--ctu-ast-mode',
                                   action='store',
                                   dest='ctu_ast_mode',
@@ -506,7 +527,7 @@ is called.""")
                                        "phase of the analysis. (default: "
                                        "parse-on-demand)")
 
-    if analyzer_types.is_statistics_capable(context):
+    if analyzer_types.is_statistics_capable():
         stat_opts = parser.add_argument_group(
             "Statistics analysis feature arguments",
             """
@@ -574,30 +595,27 @@ statisticsCollector.SpecialReturnValue checkers are available).""")
         """
 Checkers
 ------------------------------------------------
-The analyzer performs checks that are categorized into families or "checkers".
-See 'CodeChecker checkers' for the list of available checkers. You can
-fine-tune which checkers to use in the analysis by setting the enabled and
-disabled flags starting from the bigger groups and going inwards, e.g.
-'-e core -d core.uninitialized -e core.uninitialized.Assign' will enable every
-'core' checker, but only 'core.uninitialized.Assign' from the
-'core.uninitialized' group. Please consult the manual for details. Disabling
-certain checkers - such as the 'core' group - is unsupported by the LLVM/Clang
-community, and thus discouraged.
+An analyzer checks the source code with the help of checkers. Checkers
+implement a specific rule, such as "don't divide by zero", and emit a warning
+if the corresponding rule is violated. Available checkers can be listed by
+'CodeChecker checkers'.
+
+Checkers are grouped by CodeChecker via labels (described below), and sometimes
+by their analyzer tool. An example for the latter is 'clangsa', which orders
+checkers in a package hierarchy (e.g. in 'core.uninitialized.Assign', 'core'
+and 'core.uninitialized' are packages).
 
 Compiler warnings and errors
 ------------------------------------------------
 Compiler warnings are diagnostic messages that report constructions that are
 not inherently erroneous but that are risky or suggest there may have been an
-error. Compiler warnings are named 'clang-diagnostic-<warning-option>', e.g.
-Clang warning controlled by '-Wliteral-conversion' will be reported with check
-name 'clang-diagnostic-literal-conversion'. You can fine-tune which warnings to
-use in the analysis by setting the enabled and disabled flags starting from the
-bigger groups and going inwards, e.g. '-e Wunused -d Wno-unused-parameter' will
-enable every 'unused' warnings except 'unused-parameter'. These flags should
-start with a capital 'W' or 'Wno-' prefix followed by the waning name (E.g.:
-'-e Wliteral-conversion', '-d Wno-literal-conversion'). By default '-Wall' and
-'-Wextra' warnings are enabled. For more information see:
-https://clang.llvm.org/docs/DiagnosticsReference.html.
+error. However, CodeChecker views them as regular checkers.
+
+Compiler warning names are transformed by CodeChecker to reflect the analyzer
+name. For example, '-Wliteral-conversion' from clang-tidy is transformed to
+'clang-diagnostic-literal-conversion'. However, they need to be enabled by
+their original name, e.g. '-e Wliteral-conversion'.
+
 Sometimes GCC is more permissive than Clang, so it is possible that a specific
 construction doesn't compile with Clang but compiles with GCC. These
 compiler errors are also collected as CodeChecker reports as
@@ -607,20 +625,38 @@ was emitted by clang-tidy.
 
 Checker labels
 ------------------------------------------------
-In CodeChecker there is a manual grouping of checkers. These groups are
-determined by labels. The collection of labels is found in
-config/labels directory. The goal of these labels is that you can
-enable or disable checkers by these labels. See the --label flag of
-"CodeChecker checkers" command.
+Each checker is assigned several '<label>:<value>' pairs. For instance,
+'cppcheck-deallocret' has the labels 'profile:default' and 'severity:HIGH'. The
+goal of labels is that you can enable or disable a batch of checkers with them.
+
+You can enable/disable checkers belonging to a label: '-e <label>:<value>',
+e.g. '-e profile:default'.
+
+See "CodeChecker checkers --help" to learn more.
 
 Guidelines
 ------------------------------------------------
-There are several coding guidelines like CppCoreGuideline, SEI-CERT, etc. These
-are collections of best programming practices to avoid common programming
-errors. Some checkers cover the rules of these guidelines. In CodeChecker there
-is a mapping between guidelines and checkers. This way you can list and enable
+CodeChecker recognizes several third party coding guidelines, such as
+CppCoreGuidelines, SEI-CERT, or MISRA. These are collections of best
+programming practices to avoid common programming errors. Some checkers cover
+the rules of these guidelines. CodeChecker assigns the 'guideline' label to
+these checkers, such as 'guideline:sei-cert'. This way you can list and enable
 those checkers which check the fulfillment of certain guideline rules. See the
-output of "CodeChecker checkers --guideline" command.""")
+output of "CodeChecker checkers --guideline" command.
+
+Guidelines are labels themselves, and can be used as a label:
+'-e guideline:<value>', e.g. '-e guideline:sei-cert'.
+
+Batch enabling/disabling checkers
+------------------------------------------------
+You can fine-tune which checkers to use in the analysis by setting the enable
+and disable flags starting from the bigger groups and going inwards. Taking
+for example the package hierarchy of 'clangsa', '-e core -d core.uninitialized
+-e core.uninitialized.Assign' will enable every 'core' checker, but only
+'core.uninitialized.Assign' from the 'core.uninitialized' group. Mind that
+disabling certain checkers - such as the 'core' group is unsupported by the
+LLVM/Clang community, and thus discouraged.
+""")
 
     checkers_opts.add_argument('-e', '--enable',
                                dest="enable",
@@ -679,6 +715,26 @@ output of "CodeChecker checkers --guideline" command.""")
                                     "the analysis. USE WISELY AND AT YOUR "
                                     "OWN RISK!")
 
+    checkers_opts.add_argument('--disable-all',
+                               dest="disable_all",
+                               required=False,
+                               default=argparse.SUPPRESS,
+                               action='store_true',
+                               help="Disable all checkers of all analyzers. "
+                                    "It is equivalent to using \"--disable "
+                                    "default\" as the first argument.")
+
+    checkers_opts.add_argument('--no-missing-checker-error',
+                               dest="no_missing_checker_error",
+                               action='store_true',
+                               required=False,
+                               default=argparse.SUPPRESS,
+                               help="Emit a warning instead of an error when "
+                                    "an unknown checker name is given to "
+                                    "either --enable, --disable,"
+                                    "--analyzer-config and/or "
+                                    "--checker-config.")
+
     output_opts = parser.add_argument_group("output arguments")
 
     output_opts.add_argument('--print-steps',
@@ -802,7 +858,7 @@ def main(args):
 
         # --- Step 2.: Perform the analysis.
         analyze_args = argparse.Namespace(
-            logfile=logfile,
+            input=logfile,
             output_path=output_dir,
             output_format='plist',
             jobs=args.jobs,
@@ -817,9 +873,9 @@ def main(args):
                           'files',
                           'analyzers',
                           'add_compiler_defaults',
+                          'cppcheck_args_cfg_file',
                           'clangsa_args_cfg_file',
                           'tidy_args_cfg_file',
-                          'tidy_config',
                           'analyzer_config',
                           'checker_config',
                           'capture_analysis_output',
@@ -833,8 +889,11 @@ def main(args):
                           'stats_relevance_threshold',
                           'stats_min_sample_count',
                           'enable_all',
+                          'disable_all',
+                          'no_missing_checker_error',
                           'ordered_checkers',  # --enable and --disable.
                           'timeout',
+                          'review_status_config',
                           'compile_uniqueing',
                           'report_hash',
                           'enable_z3',
@@ -844,6 +903,7 @@ def main(args):
         if 'clean' in args:
             setattr(analyze_args, 'clean', True)
         __update_if_key_exists(args, analyze_args, 'verbose')
+        __update_if_key_exists(args, analyze_args, 'no_missing_checker_error')
 
         import codechecker_analyzer.cmd.analyze as analyze_module
         LOG.debug("Calling ANALYZE with args:")
