@@ -13,7 +13,6 @@ List the checkers available in the analyzers.
 import argparse
 import json
 import os
-import subprocess
 import sys
 from collections import defaultdict
 from typing import Dict, Iterable, List, Tuple
@@ -22,72 +21,15 @@ from codechecker_report_converter import twodim
 
 from codechecker_analyzer import analyzer_context
 from codechecker_analyzer.analyzers import analyzer_types
-from codechecker_analyzer.analyzers.clangsa.analyzer import ClangSA
-from codechecker_analyzer.analyzers.clangtidy.analyzer import ClangTidy
 
 from codechecker_common import arg, logger
 from codechecker_common.output import USER_FORMATS
 from codechecker_common.checker_labels import CheckerLabels
-from codechecker_analyzer import env
 from codechecker_analyzer.analyzers.config_handler import CheckerState
 
 LOG = logger.get_logger('system')
 
 
-def get_diagtool_bin():
-    """
-    Return full path of diagtool.
-
-    Select clang binary, check for a 'diagtool' binary next to the selected
-    clang binary and return full path of this binary if it exists.
-    """
-    context = analyzer_context.get_context()
-    clang_bin = context.analyzer_binaries.get(ClangSA.ANALYZER_NAME)
-
-    if not clang_bin:
-        return None
-
-    # Resolve symlink.
-    clang_bin = os.path.realpath(clang_bin)
-
-    # Find diagtool next to the clang binary.
-    diagtool_bin = os.path.join(os.path.dirname(clang_bin), 'diagtool')
-    if os.path.exists(diagtool_bin):
-        return diagtool_bin
-
-    LOG.debug("'diagtool' can not be found next to the clang binary (%s)!",
-              clang_bin)
-
-
-def get_warnings(env=None):
-    """
-    Returns list of warning flags by using diagtool.
-    """
-    diagtool_bin = get_diagtool_bin()
-
-    if not diagtool_bin:
-        return []
-
-    try:
-        result = subprocess.check_output(
-            [diagtool_bin, 'tree'],
-            env=env,
-            universal_newlines=True,
-            encoding="utf-8",
-            errors="ignore")
-        return [w[2:] for w in result.split() if w.startswith("-W")]
-    except subprocess.CalledProcessError as exc:
-        LOG.error("'diagtool' encountered an error while retrieving the "
-                  "checker list. If you are using a custom compiled clang, "
-                  "you may have forgotten to build the 'diagtool' target "
-                  "alongside 'clang' and 'clang-tidy'! Error message: %s",
-                  exc.output)
-
-        raise
-    except OSError:
-        raise
-
-
 def get_argparser_ctor_args():
     """
     This method returns a dict containing the kwargs for constructing an
@@ -160,13 +102,12 @@ def add_arguments_to_parser(parser):
                         help="Show checkers only from the analyzers "
                              "specified.")
 
-    if get_diagtool_bin():
-        parser.add_argument('-w', '--warnings',
-                            dest='show_warnings',
-                            default=argparse.SUPPRESS,
-                            action='store_true',
-                            required=False,
-                            help="Show available warning flags.")
+    parser.add_argument('-w', '--warnings',
+                        dest='show_warnings',
+                        default=argparse.SUPPRESS,
+                        action='store_true',
+                        required=False,
+                        help="DEPRECATED. Show available warning flags.")
 
     parser.add_argument('--details',
                         dest='details',
@@ -229,28 +170,6 @@ def add_arguments_to_parser(parser):
                              "These can be given to 'CodeChecker analyze "
                              "--checker-config'.")
 
-    filters = parser.add_mutually_exclusive_group(required=False)
-
-    # --only-enabled and --only-disabled flags are deprecated because they're
-    # not used anymore. Earlier we listed all checkers that would be enabled
-    # during analysis with the given filter set (--label, --profile,
-    # --guideline) and that list contained default checkers among others which
-    # were not explicitly enabled. (These could have been eliminated by
-    # --only-enabled). Now we list only those checkers which fulfill the
-    # conjunction of the provided labels, or we list all checkers if no filter
-    # is given.
-    filters.add_argument('--only-enabled',
-                         dest='only_enabled',
-                         default=argparse.SUPPRESS,
-                         action='store_true',
-                         help="DEPRECATED. Show only the enabled checkers.")
-
-    filters.add_argument('--only-disabled',
-                         dest='only_disabled',
-                         default=argparse.SUPPRESS,
-                         action='store_true',
-                         help="DEPRECATED. Show only the disabled checkers.")
-
     parser.add_argument('-o', '--output',
                         dest='output_format',
                         required=False,
@@ -270,26 +189,6 @@ def __uglify(text: str) -> str:
     return text.lower().replace(' ', '_')
 
 
-def guideline_rules_for_checker(
-    checker: str,
-    context: analyzer_context.Context
-) -> Dict[str, list]:
-    """
-    Returns the guideline and rules covered by the given checker. This
-    function returns a dict which maps guideline names to the covered
-    rules: { "sei-cert": ["arr30-c", ...], ... }.
-    """
-    result = defaultdict(list)
-    labels = context.checker_labels.labels_of_checker(checker)
-    guidelines = context.checker_labels.get_description('guideline')
-
-    for label in labels:
-        if label[0] in guidelines:
-            result[label[0]].append(label[1])
-
-    return result
-
-
 def __guideline_to_label(
     args: argparse.Namespace,
     cl: CheckerLabels
@@ -323,18 +222,11 @@ def __get_detailed_checker_info(
     supported checkers. Checker information is described with tuples of this
     information: (status, checker name, analyzer name, description, labels).
     """
-    context = analyzer_context.get_context()
-
     working_analyzers, _ = analyzer_types.check_supported_analyzers(
-        analyzer_types.supported_analyzers,
-        context)
+        analyzer_types.supported_analyzers)
 
     analyzer_config_map = analyzer_types.build_config_handlers(
-        args, context, working_analyzers)
-
-    analyzer_environment = env.extend(
-        context.path_env_extra,
-        context.ld_lib_path_extra)
+        args, working_analyzers)
 
     checker_info = defaultdict(list)
 
@@ -342,8 +234,7 @@ def __get_detailed_checker_info(
         config_handler = analyzer_config_map.get(analyzer)
         analyzer_class = analyzer_types.supported_analyzers[analyzer]
 
-        checkers = analyzer_class.get_analyzer_checkers(
-            config_handler, analyzer_environment)
+        checkers = analyzer_class.get_analyzer_checkers()
 
         profile_checkers = []
         if 'profile' in args:
@@ -366,29 +257,14 @@ def __get_detailed_checker_info(
         if 'guideline' in args:
             profile_checkers.append((__guideline_to_label(args, cl), True))
 
-        config_handler.initialize_checkers(
-            context, checkers, profile_checkers)
+        config_handler.initialize_checkers(checkers, profile_checkers)
 
         for checker, (state, description) in config_handler.checks().items():
-            # severity = cl.severity(checker)
-            # guideline = guideline_rules_for_checker(checker, context)
-            # checker_info[analyzer].append(
-            #     (state, checker, analyzer, severity, guideline, description))
+            labels = cl.labels_of_checker(checker, analyzer)
+            state = CheckerState.enabled if ('profile', 'default') in labels \
+                else CheckerState.disabled
             checker_info[analyzer].append(
-                (state, checker, analyzer, description,
-                 sorted(cl.labels_of_checker(checker, analyzer))))
-
-    if 'show_warnings' in args:
-        for warning in get_warnings(analyzer_environment):
-            warning = 'clang-diagnostic-' + warning
-            # guideline = guideline_rules_for_checker(warning, context)
-            # checker_info[ClangTidy.ANALYZER_NAME].append(
-            #     (CheckerState.default, warning, ClangTidy.ANALYZER_NAME,
-            #      'MEDIUM', guideline, ''))
-            checker_info[ClangTidy.ANALYZER_NAME].append(
-                (CheckerState.default, warning, ClangTidy.ANALYZER_NAME, '',
-                 sorted(cl.labels_of_checker(
-                    warning, ClangTidy.ANALYZER_NAME))))
+                (state, checker, analyzer, description, sorted(labels)))
 
     return checker_info
 
@@ -511,10 +387,7 @@ def __format_row(row: Tuple) -> Tuple:
     row -- A tuple with detailed checker info coming from
            __get_detailed_checker_info() function.
     """
-    state = '+' if row[0] == CheckerState.enabled else \
-        '-' if row[0] == CheckerState.disabled else \
-        '?'
-
+    state = '+' if row[0] == CheckerState.enabled else '-'
     labels = ', '.join(f'{k}:{v}' for k, v in row[4])
 
     return state, row[1], row[2], row[3], labels
@@ -530,8 +403,6 @@ def __print_checkers_custom_format(checkers: Iterable):
             status = 'enabled'
         elif checker[0] == CheckerState.disabled:
             status = 'disabled'
-        else:
-            status = 'unknown'
 
         print(checker[1])
         print('  Status:', status)
@@ -651,18 +522,10 @@ def __print_checker_config(args: argparse.Namespace):
     if args.output_format == 'custom':
         args.output_format = 'rows'
 
-    context = analyzer_context.get_context()
     working_analyzers, errored = analyzer_types.check_supported_analyzers(
-        args.analyzers,
-        context)
+        args.analyzers)
     analyzer_types.check_available_analyzers(working_analyzers, errored)
 
-    analyzer_environment = env.extend(context.path_env_extra,
-                                      context.ld_lib_path_extra)
-
-    analyzer_config_map = analyzer_types.build_config_handlers(
-        args, context, working_analyzers)
-
     if 'details' in args:
         header = ['Option', 'Description']
     else:
@@ -674,16 +537,18 @@ def __print_checker_config(args: argparse.Namespace):
     rows = []
     analyzer_failures = []
     for analyzer in working_analyzers:
-        config_handler = analyzer_config_map.get(analyzer)
         analyzer_class = analyzer_types.supported_analyzers[analyzer]
 
-        configs = analyzer_class.get_checker_config(config_handler,
-                                                    analyzer_environment)
+        configs = analyzer_class.get_checker_config()
         if not configs:
             # Checker configurations are not supported by cppcheck
             if analyzer != "cppcheck":
                 analyzer_failures.append(analyzer)
                 continue
+            # TODO
+            if analyzer != "gcc":
+                analyzer_failures.append(analyzer)
+                continue
 
         rows.extend((':'.join((analyzer, c[0])), c[1]) if 'details' in args
                     else (':'.join((analyzer, c[0])),) for c in configs)
