@@ -15,9 +15,9 @@ import argparse
 import os
 import sys
 from typing import Dict, Optional, Set
+import fnmatch
 
-from codechecker_report_converter.util import dump_json_output, \
-    load_json_or_empty
+from codechecker_report_converter.util import dump_json_output
 from codechecker_report_converter.report import report_file, \
     reports as reports_helper
 from codechecker_report_converter.report.output import baseline, codeclimate, \
@@ -25,15 +25,17 @@ from codechecker_report_converter.report.output import baseline, codeclimate, \
 from codechecker_report_converter.report.output.html import \
     html as report_to_html
 from codechecker_report_converter.report.statistics import Statistics
-from codechecker_report_converter.source_code_comment_handler import \
-    REVIEW_STATUS_VALUES
 
 
 from codechecker_analyzer import analyzer_context, suppress_handler
 
 from codechecker_common import arg, logger, cmd_config
+from codechecker_common.review_status_handler import ReviewStatusHandler
 from codechecker_common.skiplist_handler import SkipListHandler, \
     SkipListHandlers
+from codechecker_common.source_code_comment_handler import \
+    REVIEW_STATUS_VALUES
+from codechecker_common.util import load_json
 
 
 LOG = logger.get_logger('system')
@@ -257,7 +259,7 @@ def get_metadata(dir_path: str) -> Optional[Dict]:
     """ Get metadata from the given dir path or None if not exists. """
     metadata_file = os.path.join(dir_path, "metadata.json")
     if os.path.exists(metadata_file):
-        return load_json_or_empty(metadata_file)
+        return load_json(metadata_file)
 
     return None
 
@@ -281,6 +283,14 @@ def main(args):
         LOG.error(fnerr)
         sys.exit(1)
 
+    incorrect_input_paths = [input_path for input_path in args.input
+                             if not os.path.exists(input_path)]
+
+    if incorrect_input_paths:
+        for input_path in incorrect_input_paths:
+            LOG.error(f"Input path {input_path} does not exist!")
+        sys.exit(1)
+
     export = args.export if 'export' in args else None
     if export == 'html' and 'output_path' not in args:
         LOG.error("Argument --export not allowed without argument --output "
@@ -383,6 +393,7 @@ def main(args):
     processed_path_hashes = set()
     processed_file_paths = set()
     print_steps = 'print_steps' in args
+    review_status_handler = ReviewStatusHandler()
 
     html_builder: Optional[report_to_html.HtmlBuilder] = None
     if export == 'html':
@@ -391,11 +402,50 @@ def main(args):
             context.checker_labels)
 
     for dir_path, file_paths in report_file.analyzer_result_files(args.input):
+        review_status_cfg = os.path.join(dir_path, 'review_status.yaml')
+        if os.path.lexists(review_status_cfg):
+            try:
+                review_status_handler.set_review_status_config(
+                    review_status_cfg)
+            except ValueError as err:
+                LOG.error(err)
+                sys.exit(1)
+
         metadata = get_metadata(dir_path)
+
+        if metadata and 'files' in args:
+            # Mapping plists when files are specified to speed up parsing
+            # The specifed_file_paths variable would be an empty list
+            # if metadata.json did not contain the specified file or
+            # metadata did not contain mapping between source files and plists.
+            specifed_file_paths = [
+                key for key, val in
+                metadata['tools'][0]['result_source_files'].items()
+                if any(fnmatch.fnmatch(val, f) for f in args.files)
+            ] if 'tools' in metadata \
+                and len(metadata['tools']) > 0 \
+                and 'result_source_files' in metadata['tools'][0] \
+                else []
+            file_paths = specifed_file_paths or file_paths
+
         for file_path in file_paths:
             reports = report_file.get_reports(
                 file_path, context.checker_labels, file_cache)
 
+            for report in reports:
+                try:
+                    # TODO: skip_handler is used later in reports_helper.skip()
+                    # too. However, skipped reports shouldn't check source code
+                    # comments because they potentially raise an exception.
+                    # Skipped files shouldn't raise an exception, also, "skip"
+                    # shouldn't be checked twice.
+                    if not report.skip(skip_handlers):
+                        report.review_status = \
+                            review_status_handler.get_review_status(report)
+                except ValueError as err:
+                    LOG.error(err)
+                    sys.exit(1)
+
             reports = reports_helper.skip(
                 reports, processed_path_hashes, skip_handlers, suppr_handler,
                 src_comment_status_filter)
@@ -417,6 +467,7 @@ def main(args):
                 file_report_map = plaintext.get_file_report_map(
                     reports, file_path, metadata)
                 plaintext.convert(
+                    review_status_handler,
                     file_report_map, processed_file_paths, print_steps)
             elif export == 'html':
                 print(f"Parsing input file '{file_path}'.")
@@ -424,6 +475,9 @@ def main(args):
                     file_path, reports, output_dir_path,
                     html_builder)
 
+    for warning in review_status_handler.source_comment_warnings():
+        LOG.warning(warning)
+
     if export is None:  # Plain text output
         statistics.write()
     elif export == 'html':
