@@ -29,7 +29,8 @@ from codechecker_common.output import USER_FORMATS
 DEFAULT_FILTER_VALUES = {
     'review_status': ['unreviewed', 'confirmed'],
     'detection_status': ['new', 'reopened', 'unresolved'],
-    'uniqueing': 'off'
+    'uniqueing': 'off',
+    'anywhere_on_report_path': False
 }
 
 DEFAULT_OUTPUT_FORMATS = ["plaintext"] + USER_FORMATS
@@ -424,29 +425,14 @@ def __add_filtering_arguments(parser, defaults=None, diff_mode=False):
                               "\"time\" part can be omitted, in which case "
                               "midnight (00:00:00) is used).")
 
-    f_group.add_argument('-s', '--suppressed',
-                         default=argparse.SUPPRESS,
-                         dest="suppressed",
-                         action='store_true',
-                         help="DEPRECATED. Use the '--filter' option to get "
-                              "false positive (suppressed) results. Show only "
-                              "suppressed results instead of only "
-                              "unsuppressed ones.")
-
-    f_group.add_argument('--filter',
-                         type=str,
-                         dest='filter',
-                         default=argparse.SUPPRESS,
-                         help="DEPRECATED. Filter results. Use separated "
-                              "filter options to filter the results. The "
-                              "filter string has the following format: "
-                              "[<SEVERITIES>]:[<CHECKER_NAMES>]:"
-                              "[<FILE_PATHS>]:[<DETECTION_STATUSES>]:"
-                              "[<REVIEW_STATUSES>] where severites, "
-                              "checker_names, file_paths, detection_statuses, "
-                              "review_statuses should be a comma separated "
-                              "list, e.g.: \"high,medium:unix,core:*.cpp,*.h:"
-                              "new,unresolved:false_positive,intentional\"")
+    f_group.add_argument('--anywhere-on-report-path',
+                         dest='anywhere_on_report_path',
+                         required=False,
+                         default=init_default('anywhere_on_report_path'),
+                         action="store_true",
+                         help="Filter reports where the report path not only "
+                              "ends in the files given by --file or "
+                              "--component, but goes through them.")
 
 
 def __register_results(parser):
@@ -489,6 +475,7 @@ def __register_diff(parser):
                         type=str,
                         nargs='+',
                         dest="base_run_names",
+                        required=True,
                         metavar='BASE_RUNS',
                         default=argparse.SUPPRESS,
                         help="The 'base' (left) side of the difference: these "
@@ -512,6 +499,7 @@ def __register_diff(parser):
                         type=str,
                         nargs='+',
                         dest="new_run_names",
+                        required=True,
                         metavar='NEW_RUNS',
                         default=argparse.SUPPRESS,
                         help="The 'new' (right) side of the difference: these "
@@ -618,17 +606,6 @@ def __register_sum(parser):
                             default=argparse.SUPPRESS,
                             help="Show breakdown for all analysis runs.")
 
-    parser.add_argument('--disable-unique',
-                        dest="disable_unique",
-                        action='store_true',
-                        default=argparse.SUPPRESS,
-                        help="DEPRECATED. Use the '--uniqueing' option to "
-                             "get uniqueing results. List all bugs even if "
-                             "these end up in the same bug location, but "
-                             "reached through different paths. By uniqueing "
-                             "the bugs a report will be appeared only once "
-                             "even if it is found on several paths.")
-
     default_filter_values = DEFAULT_FILTER_VALUES
     default_filter_values['uniqueing'] = 'on'
     __add_filtering_arguments(parser, default_filter_values)
@@ -776,6 +753,15 @@ def __register_products(parser):
                             help="A custom textual description to be shown "
                                  "alongside the product.")
 
+        parser.add_argument('--report-limit',
+                            type=int,
+                            dest="report_limit",
+                            default=argparse.SUPPRESS,
+                            required=False,
+                            help="The maximum number of reports allowed to "
+                                 "store in one run, if exceeded, the store "
+                                 "action will be rejected.")
+
         dbmodes = parser.add_argument_group(
             "database arguments",
             "NOTE: These database arguments are relative to the server "
