@@ -17,13 +17,18 @@ import functools
 import hashlib
 import json
 import os
+import signal
 import sys
 import tempfile
+import uuid
 import zipfile
 import zlib
+import shutil
 
 from collections import defaultdict, namedtuple
-from concurrent.futures import ProcessPoolExecutor
+from contextlib import contextmanager
+from datetime import timedelta
+from threading import Timer
 from typing import Dict, Iterable, List, Set, Tuple
 
 from codechecker_api.codeCheckerDBAccess_v6.ttypes import StoreLimitKind
@@ -32,14 +37,18 @@ from codechecker_api_shared.ttypes import RequestFailed, ErrorCode
 from codechecker_report_converter import twodim
 from codechecker_report_converter.report import Report, report_file, \
     reports as reports_helper, statistics as report_statistics
-from codechecker_report_converter.report.hash import HashType
-from codechecker_report_converter.source_code_comment_handler import \
-    SourceCodeCommentHandler
-from codechecker_report_converter.util import load_json_or_empty
+from codechecker_report_converter.report.hash import HashType, \
+    get_report_path_hash
+from codechecker_report_converter.report.parser.base import AnalyzerInfo
 
 from codechecker_client import client as libclient
+from codechecker_client import product
 from codechecker_common import arg, logger, cmd_config
 from codechecker_common.checker_labels import CheckerLabels
+from codechecker_common.compatibility.multiprocessing import Pool
+from codechecker_common.source_code_comment_handler import \
+    SourceCodeCommentHandler
+from codechecker_common.util import load_json
 
 from codechecker_web.shared import webserver_context, host_check
 from codechecker_web.shared.env import get_default_workspace
@@ -47,7 +56,13 @@ from codechecker_web.shared.env import get_default_workspace
 try:
     from codechecker_client.blame_info import assemble_blame_info
 except ImportError:
-    pass
+    def assemble_blame_info(_, __) -> int:
+        """
+        Shim for cases where Git blame info is not gatherable due to
+        missing libraries.
+        """
+        raise NotImplementedError()
+
 
 LOG = logger.get_logger('system')
 
@@ -299,7 +314,7 @@ def __get_run_name(input_list):
     for input_path in input_list:
         metafile = os.path.join(input_path, "metadata.json")
         if os.path.isdir(input_path) and os.path.exists(metafile):
-            metajson = load_json_or_empty(metafile)
+            metajson = load_json(metafile)
 
             if 'version' in metajson and metajson['version'] >= 2:
                 for tool in metajson.get('tools', {}):
@@ -365,16 +380,8 @@ def filter_source_files_with_comments(
     """
     jobs = file_report_positions.items()
 
-    # Currently ProcessPoolExecutor fails completely in windows.
-    # Reason is most likely combination of venv and fork() not
-    # being present in windows, so stuff like setting up
-    # PYTHONPATH in parent CodeChecker before store is executed
-    # are lost.
-    if sys.platform == "win32":
-        return get_source_file_with_comments(jobs)
-    else:
-        with ProcessPoolExecutor() as executor:
-            return get_source_file_with_comments(jobs, executor.map)
+    with Pool() as executor:
+        return get_source_file_with_comments(jobs, executor.map)
 
 
 def get_reports(
@@ -406,21 +413,31 @@ def parse_analyzer_result_files(
     """ Get reports from the given analyzer result files. """
     analyzer_result_file_reports: AnalyzerResultFileReports = defaultdict(list)
 
-    for file_path, reports in zip(
+    for idx, (file_path, reports) in enumerate(zip(
             analyzer_result_files, zip_iter(
                 functools.partial(get_reports, checker_labels=checker_labels),
-                analyzer_result_files)):
+                analyzer_result_files))):
+        LOG.debug(f"[{idx}/{len(analyzer_result_files)}] "
+                  f"Parsed '{file_path}' ...")
         analyzer_result_file_reports[file_path] = reports
 
     return analyzer_result_file_reports
 
 
-def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
+def assemble_zip(inputs,
+                 zip_file,
+                 client,
+                 prod_client,
+                 checker_labels: CheckerLabels):
     """Collect and compress report and source files, together with files
     contanining analysis related information into a zip file which
     will be sent to the server.
+
+    For each report directory, we create a uniqued zipped directory. Each
+    report directory to store could have been made with different
+    configurations, so we can't merge them all into a single zip.
     """
-    files_to_compress = set()
+    files_to_compress: Dict[str, set] = defaultdict(set)
     analyzer_result_file_paths = []
     stats = StorageZipStatistics()
 
@@ -429,7 +446,8 @@ def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
 
         metadata_file_path = os.path.join(dir_path, 'metadata.json')
         if os.path.exists(metadata_file_path):
-            files_to_compress.add(metadata_file_path)
+            files_to_compress[os.path.dirname(metadata_file_path)] \
+                .add(metadata_file_path)
 
         skip_file_path = os.path.join(dir_path, 'skip_file')
         if os.path.exists(skip_file_path):
@@ -437,43 +455,65 @@ def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
                 LOG.info("Found skip file %s with the following content:\n%s",
                          skip_file_path, f.read())
 
-            files_to_compress.add(skip_file_path)
+            files_to_compress[os.path.dirname(skip_file_path)] \
+                .add(skip_file_path)
 
-    LOG.debug("Processing report files ...")
+        review_status_file_path = os.path.join(dir_path, 'review_status.yaml')
+        if os.path.exists(review_status_file_path):
+            files_to_compress[os.path.dirname(review_status_file_path)]\
+                .add(review_status_file_path)
 
-    # Currently ProcessPoolExecutor fails completely in windows.
-    # Reason is most likely combination of venv and fork() not
-    # being present in windows, so stuff like setting up
-    # PYTHONPATH in parent CodeChecker before store is executed
-    # are lost.
-    if sys.platform == "win32":
+    LOG.debug(f"Processing {len(analyzer_result_file_paths)} report files ...")
+
+    with Pool() as executor:
         analyzer_result_file_reports = parse_analyzer_result_files(
-            analyzer_result_file_paths, checker_labels)
-    else:
-        with ProcessPoolExecutor() as executor:
-            analyzer_result_file_reports = parse_analyzer_result_files(
-                 analyzer_result_file_paths, checker_labels, executor.map)
+             analyzer_result_file_paths, checker_labels, executor.map)
 
     LOG.info("Processing report files done.")
 
     changed_files = set()
     file_paths = set()
     file_report_positions: FileReportPositions = defaultdict(set)
+    unique_reports: Dict[str, Dict[str, List[Report]]] = defaultdict(dict)
+
+    unique_report_hashes = set()
     for file_path, reports in analyzer_result_file_reports.items():
-        files_to_compress.add(file_path)
         stats.num_of_analyzer_result_files += 1
 
         for report in reports:
             if report.changed_files:
                 changed_files.update(report.changed_files)
                 continue
-            stats.add_report(report)
+            # Unique all bug reports per report directory; also, count how many
+            # reports we want to store at once to check for the report store
+            # limit.
+            report_path_hash = get_report_path_hash(report)
+            if report_path_hash not in unique_report_hashes:
+                unique_report_hashes.add(report_path_hash)
+                unique_reports[os.path.dirname(file_path)]\
+                    .setdefault(report.analyzer_name, []) \
+                    .append(report)
+                stats.add_report(report)
 
             file_paths.update(report.original_files)
             file_report_positions[report.file.original_path].add(report.line)
 
+    temp_dir = tempfile.mkdtemp('-unique-plists', dir=inputs[0])
+    for dirname, analyzer_reports in unique_reports.items():
+        for analyzer_name, reports in analyzer_reports.items():
+            if not analyzer_name:
+                analyzer_name = 'unknown'
+            tmpfile = os.path.join(
+                temp_dir, f'{uuid.uuid4()}-{analyzer_name}.plist')
+
+            report_file.create(tmpfile, reports, checker_labels,
+                               AnalyzerInfo(analyzer_name))
+            LOG.debug(f"Stored '{analyzer_name}' unique reports in {tmpfile}.")
+            files_to_compress[dirname].add(tmpfile)
+
     if changed_files:
         reports_helper.dump_changed_files(changed_files)
+        shutil.rmtree(temp_dir)
         sys.exit(1)
 
     if not file_paths:
@@ -527,15 +567,17 @@ def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
     LOG.info("Building report zip file...")
     with zipfile.ZipFile(zip_file, 'a', allowZip64=True) as zipf:
         # Add the files to the zip which will be sent to the server.
-        for file_path in files_to_compress:
-            _, file_name = os.path.split(file_path)
 
-            # Create a unique report directory name.
-            report_dir_name = hashlib.md5(os.path.dirname(
-                file_path).encode('utf-8')).hexdigest()
+        for dirname, files in files_to_compress.items():
+            for file_path in files:
+                _, file_name = os.path.split(file_path)
 
-            zip_target = os.path.join('reports', report_dir_name, file_name)
-            zipf.write(file_path, zip_target)
+                # Create a unique report directory name.
+                report_dir_name = \
+                    hashlib.md5(dirname.encode('utf-8')).hexdigest()
+                zip_target = \
+                    os.path.join('reports', report_dir_name, file_name)
+                zipf.write(file_path, zip_target)
 
         collected_file_paths = set()
         for f, h in file_to_hash.items():
@@ -560,14 +602,13 @@ def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
                     zipf, file_paths)
 
                 if stats.num_of_blame_information:
-                    LOG.info("Collecting blame information done.")
+                    LOG.info("Collecting blame information... Done.")
                 else:
                     LOG.info("No blame information found for source files.")
-            except NameError:
+            except NotImplementedError:
                 LOG.warning(
-                    "Collecting blame information has been failed. Make sure "
-                    "'git' is available on your system to hide this warning "
-                    "message.")
+                    "Failed to collect blame information. Make sure Git is "
+                    "installed on your system.")
 
         zipf.writestr('content_hashes.json', json.dumps(file_to_hash))
 
@@ -576,6 +617,26 @@ def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
     # Print statistics what will be stored to the server.
     stats.write()
 
+    # Fail store early if too many reports.
+    p = prod_client.getCurrentProduct()
+    if len(unique_reports) > p.reportLimit:
+        LOG.error(f"""Report Limit Exceeded
+
+This report folder cannot be stored because the number of reports in the
+result folder is too high. Usually noisy checkers, generating a lot of
+reports are not useful and it is better to disable them.
+
+Run `CodeChecker parse <report_folder>` to gain a better understanding on
+report counts.
+
+Disable checkers that have generated an excessive number of reports and then
+rerun the analysis to be able to store the results on the server.
+
+Configured report limit for this product: {p.reportLimit}
+        """)
+        shutil.rmtree(temp_dir)
+        sys.exit(1)
+
     zip_size = os.stat(zip_file).st_size
 
     LOG.info("Compressing report zip file...")
@@ -590,6 +651,9 @@ def assemble_zip(inputs, zip_file, client, checker_labels: CheckerLabels):
     LOG.info("Compressing report zip file done (%s / %s).",
              sizeof_fmt(zip_size), sizeof_fmt(compressed_zip_size))
 
+    # We are responsible for deleting these.
+    shutil.rmtree(temp_dir)
+
 
 def should_be_zipped(input_file: str, input_files: Iterable[str]) -> bool:
     """
@@ -713,6 +777,71 @@ def storing_analysis_statistics(client, inputs, run_name):
         os.remove(zip_file)
 
 
+class WatchdogError(Exception):
+    """
+    Custom exception type thrown by '_timeout_watchdog'. This is used instead
+    of the built-in 'TimeoutError' because TimeoutError <: OSError and other
+    handlers in the call chain during a timeout might catch the more generic
+    OSError instead, resulting in not running the appropriate handler.
+    """
+    def __init__(self, timeout: timedelta, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.timeout = timeout
+
+
+@contextmanager
+def _timeout_watchdog(timeout: timedelta, trap: int):
+    """
+    Sets up a context with a 'threading.Timer' which will signal 'trap' the
+    current process if 'timeout' passes. If and only if this 'trap' signal
+    arrives, a 'WatchdogError' is thrown for appropriate handling outside the
+    context.
+
+    The timer is started immediately as this function returns the context.
+
+    This is done in order to prevent the current process from hanging
+    indefinitely in case the stored data is too big and the server spends more
+    time processing it that what the network connection would stay alive for.
+
+    The fact that this works is **NOT GUARANTEED**! In some cases, based on
+    likely the kernel version, its implementation, and the exact network
+    topology, the client may enter an *UNINTERRUPTIBLE* sleep ('D') at which
+    point signals *MAY* not be accurately received. However, manual testing
+    revealbed that resurrection of hung clients is possible even if the process
+    is shown as "uninterruptible" by simply killing it from the outside.
+
+    This is a **TEMPORARY** measure to alleviate hangs until the proper
+    "pollable asynchronous store" feature is implemented, see:
+    http://github.com/Ericsson/codechecker/issues/3672
+    """
+    def _signal_handler(sig: int, frame):
+        if sig == trap:
+            signal.signal(trap, signal.SIG_DFL)
+            raise WatchdogError(timeout,
+                                "Timeout watchdog hit %d seconds (%s)"
+                                % (timeout.total_seconds(), str(timeout)))
+
+    pid = os.getpid()
+    timer = Timer(timeout.total_seconds(),
+                  lambda: os.kill(pid, trap))
+    timer.name = "StoreTimeoutWatcher"
+    LOG.debug("Set up timer for %d seconds (%s) for PID %d",
+              timeout.total_seconds(), str(timeout), pid)
+    try:
+        signal.signal(trap, _signal_handler)
+        timer.start()
+        yield timer
+    except Exception as e:
+        LOG.debug("External exception within timeout-sensing context: %s",
+                  str(e))
+        raise e
+    finally:
+        timer.cancel()
+        signal.signal(trap, signal.SIG_DFL)
+        LOG.debug("Timeout timer of %d seconds (%s) for PID %d stopped.",
+                  timeout.total_seconds(), str(timeout), pid)
+
+
 def main(args):
     """
     Store the defect results in the specified input list as bug reports in the
@@ -761,8 +890,23 @@ def main(args):
 
     # Setup connection to the remote server.
     client = libclient.setup_client(args.product_url)
+    protocol, host, port, product_name = \
+        product.split_product_url(args.product_url)
+    prod_client = libclient.setup_product_client(protocol,
+                                                 host,
+                                                 port,
+                                                 product_name=product_name)
 
-    zip_file_handle, zip_file = tempfile.mkstemp('.zip')
+    try:
+        temp_dir = tempfile.mkdtemp(suffix="-store", dir=args.input[0])
+        LOG.debug(f"{temp_dir} directory created successfully!")
+    except PermissionError:
+        LOG.error(f"Permission denied! You do not have sufficient "
+                  f"permissions to create the {temp_dir} "
+                  "temporary directory.")
+        sys.exit(1)
+
+    zip_file_handle, zip_file = tempfile.mkstemp(suffix=".zip", dir=temp_dir)
     LOG.debug("Will write mass store ZIP to '%s'...", zip_file)
 
     try:
@@ -770,7 +914,11 @@ def main(args):
 
         LOG.debug("Assembling zip file.")
         try:
-            assemble_zip(args.input, zip_file, client, context.checker_labels)
+            assemble_zip(args.input,
+                         zip_file,
+                         client,
+                         prod_client,
+                         context.checker_labels)
         except Exception as ex:
             print(ex)
             import traceback
@@ -798,13 +946,56 @@ def main(args):
 
         LOG.info("Storing results to the server...")
 
-        client.massStoreRun(args.name,
-                            args.tag if 'tag' in args else None,
-                            str(context.version),
-                            b64zip,
-                            'force' in args,
-                            trim_path_prefixes,
-                            description)
+        try:
+            with _timeout_watchdog(timedelta(hours=1),
+                                   signal.SIGUSR1):
+                client.massStoreRun(args.name,
+                                    args.tag if 'tag' in args else None,
+                                    str(context.version),
+                                    b64zip,
+                                    'force' in args,
+                                    trim_path_prefixes,
+                                    description)
+        except WatchdogError as we:
+            LOG.warning("%s", str(we))
+
+            # Showing parts of the exception stack is important here.
+            # We **WANT** to see that the timeout happened during a wait on
+            # Thrift reading from the TCP connection (something deep in the
+            # Python library code at "sock.recv_into").
+            import traceback
+            _, _, tb = sys.exc_info()
+            frames = traceback.extract_tb(tb)
+            first, last = frames[0], frames[-2]
+            formatted_frames = traceback.format_list([first, last])
+            fmt_first, fmt_last = formatted_frames[0], formatted_frames[1]
+            LOG.info("Timeout was triggered during:\n%s", fmt_first)
+            LOG.info("Timeout interrupted this low-level operation:\n%s",
+                     fmt_last)
+
+            LOG.error("Timeout!"
+                      "\n\tThe server's reply did not arrive after "
+                      "%d seconds (%s) elapsed since the server-side "
+                      "processing began."
+                      "\n\n\tThis does *NOT* mean that there was an issue "
+                      "with the run you were storing!"
+                      "\n\tThe server might still be processing the results..."
+                      "\n\tHowever, it is more likely that the "
+                      "server had already finished, but the client did not "
+                      "receive a response."
+                      "\n\tUsually, this is caused by the underlying TCP "
+                      "connection failing to signal a low-level disconnect."
+                      "\n\tClients potentially hanging indefinitely in these "
+                      "scenarios is an unfortunate and known issue."
+                      "\n\t\tSee http://github.com/Ericsson/codechecker/"
+                      "issues/3672 for details!"
+                      "\n\n\tThis error here is a temporary measure to ensure "
+                      "an infinite hang is replaced with a well-explained "
+                      "timeout."
+                      "\n\tA more proper solution will be implemented in a "
+                      "subsequent version of CodeChecker.",
+                      we.timeout.total_seconds(), str(we.timeout))
+            sys.exit(1)
 
         # Storing analysis statistics if the server allows them.
         if client.allowsStoringAnalysisStatistics():
@@ -828,3 +1019,5 @@ def main(args):
     finally:
         os.close(zip_file_handle)
         os.remove(zip_file)
+        if os.path.exists(temp_dir):
+            shutil.rmtree(temp_dir)
