@@ -26,13 +26,16 @@ if sys.version_info >= (3, 8):
 else:
     from mypy_extensions import TypedDict
 
-from codechecker_report_converter import __title__, __version__
-from codechecker_report_converter.report import BugPathEvent, \
-    BugPathPosition, File, get_or_create_file, MacroExpansion, Range, Report
+from codechecker_report_converter.report import \
+    BugPathEvent, BugPathPosition, \
+    File, \
+    MacroExpansion, \
+    Range, Report, \
+    UnknownChecker, \
+    get_or_create_file
 from codechecker_report_converter.report.hash import get_report_hash, HashType
 from codechecker_report_converter.report.parser.base import AnalyzerInfo, \
-    BaseParser
-from codechecker_report_converter.util import load_json_or_empty
+    BaseParser, get_tool_info
 
 
 LOG = logging.getLogger('report-converter')
@@ -76,10 +79,15 @@ class _LXMLPlistParser(_PlistParser):
         # To be backward compatible with old interpreters we need to call this
         # function based on conditions:
         params = _PlistParser.__init__.__code__.co_varnames
+        # Before 3.9 interpreter. When a newer interpreter is used, pylint will
+        # complain that too many arguments are used to invoke __init__, but
+        # with newer interpreters, this is deadcode.
         if len(params) == 3 and "use_builtin_types" in params:
             # Before 3.9 interpreter.
+            # pylint: disable=E1121
             _PlistParser.__init__(self, True, dict_type)
         else:
+            # After 3.9 interpreter.
             _PlistParser.__init__(self, dict_type)  # pylint: disable=E1120
 
         self.event_handler = _LXMLPlistEventHandler()
@@ -239,11 +247,14 @@ class Parser(BaseParser):
         files: Dict[int, File],
         metadata: Dict[str, Any]
     ) -> Report:
-        location = diag.get('location', {})
-        checker_name = diag.get('check_name', "unknown")
+        location = diag.get("location", {})
+        checker_name = diag.get("check_name", UnknownChecker[1])
         analyzer_name = self.__get_analyzer_name(checker_name, metadata)
         severity = self.get_severity(checker_name)
 
+        report_annotation = diag["report-annotation"] \
+            if "report-annotation" in diag else None
+
         return Report(
             analyzer_result_file_path=analyzer_result_file_path,
             file=files[location['file']],
@@ -259,7 +270,8 @@ class Parser(BaseParser):
             bug_path_events=self.__get_bug_path_events(diag, files),
             bug_path_positions=self.__get_bug_path_positions(diag, files),
             notes=self.__get_notes(diag, files),
-            macro_expansions=self.__get_macro_expansions(diag, files))
+            macro_expansions=self.__get_macro_expansions(diag, files),
+            annotations=report_annotation)
 
     def __get_analyzer_name(
         self,
@@ -413,33 +425,13 @@ class Parser(BaseParser):
 
         return macro_expansions
 
-    def __get_tool_info(self) -> Tuple[str, str]:
-        """ Get tool info.
-
-        If this was called through CodeChecker, this function will return
-        CodeChecker information, otherwise this tool (report-converter)
-        information.
-        """
-        data_files_dir_path = os.environ.get('CC_DATA_FILES_DIR')
-        if data_files_dir_path:
-            analyzer_version_file_path = os.path.join(
-                data_files_dir_path, 'config', 'analyzer_version.json')
-            if os.path.exists(analyzer_version_file_path):
-                data = load_json_or_empty(analyzer_version_file_path, {})
-                version = data.get('version')
-                if version:
-                    return 'CodeChecker', f"{version['major']}." \
-                        f"{version['minor']}.{version['revision']}"
-
-        return __title__, __version__
-
     def convert(
         self,
         reports: List[Report],
         analyzer_info: Optional[AnalyzerInfo] = None
     ):
         """ Converts the given reports. """
-        tool_name, tool_version = self.__get_tool_info()
+        tool_name, tool_version = get_tool_info()
 
         data: Dict[str, Any] = {
             'files': [],
@@ -527,6 +519,11 @@ class Parser(BaseParser):
                         self._create_macro_expansion(
                             macro_expansion, file_index_map))
 
+            if report.annotations:
+                diagnostic["report-annotation"] = dict()
+                for key, value in report.annotations.items():
+                    diagnostic["report-annotation"][key] = value
+
             data['diagnostics'].append(diagnostic)
 
         return data
