@@ -129,6 +129,8 @@ class ThriftProductHandler:
             confidentiality = \
                     confidentiality_enum(product.confidentiality)
 
+        report_limit = product.report_limit
+
         return server_product, ttypes.Product(
             id=product.id,
             endpoint=product.endpoint,
@@ -141,7 +143,8 @@ class ThriftProductHandler:
             administrating=self.__administrating(args),
             databaseStatus=server_product.db_status,
             admins=[admin.name for admin in admins],
-            confidentiality=confidentiality)
+            confidentiality=confidentiality,
+            reportLimit=report_limit)
 
     @timeit
     def getPackageVersion(self):
@@ -295,6 +298,7 @@ class ThriftProductHandler:
                 description_b64=descr,
                 connection=dbc,
                 runLimit=product.run_limit,
+                reportLimit=product.report_limit,
                 isReviewStatusChangeDisabled=is_review_status_change_disabled,
                 confidentiality=confidentiality)
 
@@ -372,7 +376,8 @@ class ThriftProductHandler:
                 msg)
 
         conn_str = SQLServer \
-            .from_cmdline_args(conn_str_args, IDENTIFIER, None, False, None) \
+            .from_cmdline_args(conn_str_args, product.endpoint, IDENTIFIER,
+                               None, False, None) \
             .get_connection_string()
 
         is_rws_change_disabled = product.isReviewStatusChangeDisabled
@@ -387,6 +392,7 @@ class ThriftProductHandler:
                 name=displayed_name,
                 description=description,
                 run_limit=product.runLimit,
+                report_limit=product.reportLimit,
                 is_review_status_change_disabled=is_rws_change_disabled,
                 confidentiality=confidentiality)
 
@@ -529,8 +535,9 @@ class ThriftProductHandler:
                     msg)
 
             conn_str = SQLServer \
-                .from_cmdline_args(conn_str_args, IDENTIFIER, None,
-                                   False, None).get_connection_string()
+                .from_cmdline_args(conn_str_args, product.endpoint,
+                                   IDENTIFIER, None, False, None) \
+                .get_connection_string()
 
             # If endpoint or database arguments change, the product
             # configuration has changed so severely, that it needs
@@ -583,6 +590,7 @@ class ThriftProductHandler:
             # Update the settings in the database.
             product.endpoint = new_config.endpoint
             product.run_limit = new_config.runLimit
+            product.report_limit = new_config.reportLimit
             product.is_review_status_change_disabled = \
                 new_config.isReviewStatusChangeDisabled
             product.connection = conn_str
