@@ -13,8 +13,10 @@ and browser requests.
 
 import atexit
 import datetime
+from functools import partial
 from hashlib import sha256
-from multiprocessing import Process
+from http.server import HTTPServer, BaseHTTPRequestHandler, \
+    SimpleHTTPRequestHandler
 import os
 import posixpath
 from random import sample
@@ -24,11 +26,10 @@ import socket
 import ssl
 import sys
 import stat
+from typing import List, Optional, Tuple
 import urllib
 
-from http.server import HTTPServer, BaseHTTPRequestHandler, \
-    SimpleHTTPRequestHandler
-
+import multiprocess
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.sql.expression import func
 from thrift.protocol import TJSONProtocol
@@ -48,17 +49,15 @@ from codechecker_api.ProductManagement_v6 import \
 from codechecker_api.ServerInfo_v6 import \
     serverInfoService as ServerInfoAPI_v6
 
+from codechecker_common import util
 from codechecker_common.logger import get_logger
+from codechecker_common.compatibility.multiprocessing import \
+    Pool, cpu_count
 
+from codechecker_web.shared import database_status
 from codechecker_web.shared.version import get_version_str
 
-from . import instance_manager
-from . import permissions
-from . import routing
-from . import session_manager
-
-from .tmp import get_tmp_dir_hash
-
+from . import instance_manager, permissions, routing, session_manager
 from .api.authentication import ThriftAuthHandler as AuthHandler_v6
 from .api.config_handler import ThriftConfigHandler as ConfigHandler_v6
 from .api.product_server import ThriftProductHandler as ProductHandler_v6
@@ -70,6 +69,8 @@ from .database.config_db_model import Product as ORMProduct, \
     Configuration as ORMConfiguration
 from .database.database import DBSession
 from .database.run_db_model import IDENTIFIER as RUN_META, Run, RunLock
+from .tmp import get_tmp_dir_hash
+
 
 LOG = get_logger('server')
 
@@ -511,14 +512,15 @@ class Product:
     # connect() call so the next could be made.
     CONNECT_RETRY_TIMEOUT = 300
 
-    def __init__(self, orm_object, context, check_env):
+    def __init__(self, id_: int, endpoint: str, display_name: str,
+                 connection_string: str, context, check_env):
         """
         Set up a new managed product object for the configuration given.
         """
-        self.__id = orm_object.id
-        self.__endpoint = orm_object.endpoint
-        self.__connection_string = orm_object.connection
-        self.__display_name = orm_object.display_name
+        self.__id = id_
+        self.__endpoint = endpoint
+        self.__display_name = display_name
+        self.__connection_string = connection_string
         self.__driver_name = None
         self.__context = context
         self.__check_env = check_env
@@ -585,11 +587,11 @@ class Product:
 
         Each time the connect is called the db_status is updated.
         """
-
         LOG.debug("Checking '%s' database.", self.endpoint)
 
         sql_server = database.SQLServer.from_connection_string(
             self.__connection_string,
+            self.__endpoint,
             RUN_META,
             self.__context.run_migration_root,
             interactive=False,
@@ -666,18 +668,85 @@ class Product:
         """
         Cleanup the run database which belongs to this product.
         """
-        LOG.info("Garbage collection for product '%s' started...",
-                 self.endpoint)
+        LOG.info("[%s] Garbage collection started...", self.endpoint)
 
-        db_cleanup.remove_expired_run_locks(self.session_factory)
-        db_cleanup.remove_unused_data(self.session_factory)
-        db_cleanup.upgrade_severity_levels(self.session_factory,
-                                           self.__context.checker_labels)
+        db_cleanup.remove_expired_data(self)
+        db_cleanup.remove_unused_data(self)
+        db_cleanup.update_contextual_data(self, self.__context)
 
-        LOG.info("Garbage collection finished.")
+        LOG.info("[%s] Garbage collection finished.", self.endpoint)
         return True
 
 
+def _do_db_cleanup(context, check_env,
+                   id_: int, endpoint: str, display_name: str,
+                   connection_str: str) -> Tuple[Optional[bool], str]:
+    # This functions is a concurrent job handler!
+    try:
+        prod = Product(id_, endpoint, display_name, connection_str,
+                       context, check_env)
+        prod.connect(init_db=False)
+        if prod.db_status != DBStatus.OK:
+            status_str = database_status.db_status_msg.get(prod.db_status)
+            return None, \
+                f"Cleanup not attempted, database status is \"{status_str}\""
+
+        prod.cleanup_run_db()
+        prod.teardown()
+
+        # Result is hard-wired to True, because the db_cleanup routines
+        # swallow and log the potential errors but do not return them.
+        return True, ""
+    except Exception as e:
+        import traceback
+        traceback.print_exc()
+        return False, str(e)
+
+
+def _do_db_cleanups(config_database, context, check_env) \
+        -> Tuple[bool, List[Tuple[str, str]]]:
+    """
+    Performs on-demand start-up database cleanup on all the products present
+    in the ``config_database``.
+
+    Returns whether database clean-up succeeded for all products, and the
+    list of products for which it failed, along with the failure reason.
+    """
+    def _get_products() -> List[Product]:
+        products = list()
+        cfg_engine = config_database.create_engine()
+        cfg_session_factory = sessionmaker(bind=cfg_engine)
+        with DBSession(cfg_session_factory) as cfg_db:
+            for row in cfg_db.query(ORMProduct) \
+                    .order_by(ORMProduct.endpoint.asc()) \
+                    .all():
+                products.append((row.id, row.endpoint, row.display_name,
+                                 row.connection))
+        cfg_engine.dispose()
+        return products
+
+    products = _get_products()
+    if not products:
+        return True, list()
+
+    thr_count = util.clamp(1, len(products), cpu_count())
+    overall_result, failures = True, list()
+    with Pool(max_workers=thr_count) as executor:
+        LOG.info("Performing database cleanup using %d concurrent jobs...",
+                 thr_count)
+        for product, result in \
+                zip(products, executor.map(
+                    partial(_do_db_cleanup, context, check_env),
+                    *zip(*products))):
+            success, reason = result
+            if not success:
+                _, endpoint, _, _ = product
+                overall_result = False
+                failures.append((endpoint, reason))
+
+    return overall_result, failures
+
+
 class CCSimpleHttpServer(HTTPServer):
     """
     Simple http server to handle requests from the clients.
@@ -691,7 +760,6 @@ class CCSimpleHttpServer(HTTPServer):
                  RequestHandlerClass,
                  config_directory,
                  product_db_sql_server,
-                 skip_db_cleanup,
                  pckg_data,
                  context,
                  check_env,
@@ -729,11 +797,6 @@ class CCSimpleHttpServer(HTTPServer):
         cfg_sess.commit()
         cfg_sess.close()
 
-        if not skip_db_cleanup:
-            for endpoint, product in self.__products.items():
-                if not product.cleanup_run_db():
-                    LOG.warning("Cleaning database for %s Failed.", endpoint)
-
         try:
             HTTPServer.__init__(self, server_address,
                                 RequestHandlerClass,
@@ -833,7 +896,10 @@ class CCSimpleHttpServer(HTTPServer):
 
         LOG.debug("Setting up product '%s'", orm_product.endpoint)
 
-        prod = Product(orm_product,
+        prod = Product(orm_product.id,
+                       orm_product.endpoint,
+                       orm_product.display_name,
+                       orm_product.connection,
                        self.context,
                        self.check_env)
 
@@ -983,7 +1049,7 @@ def __make_root_file(root_file):
 
 
 def start_server(config_directory, package_data, port, config_sql_server,
-                 listen_address, force_auth, skip_db_cleanup,
+                 listen_address, force_auth, skip_db_cleanup: bool,
                  context, check_env):
     """
     Start http server to handle web client and thrift requests.
@@ -1043,6 +1109,20 @@ def start_server(config_directory, package_data, port, config_sql_server,
         LOG.error("The server's configuration file is invalid!")
         sys.exit(1)
 
+    if not skip_db_cleanup:
+        all_success, fails = _do_db_cleanups(config_sql_server,
+                                             context,
+                                             check_env)
+        if not all_success:
+            LOG.error("Failed to perform automatic cleanup on %d products! "
+                      "Earlier logs might contain additional detailed "
+                      "reasoning.\n\t* %s", len(fails),
+                      "\n\t* ".join(
+                        ("'%s' (%s)" % (ep, reason) for (ep, reason) in fails)
+                      ))
+    else:
+        LOG.debug("Skipping db_cleanup, as requested.")
+
     server_clazz = CCSimpleHttpServer
     if ':' in server_addr[0]:
         # IPv6 address specified for listening.
@@ -1054,7 +1134,6 @@ def start_server(config_directory, package_data, port, config_sql_server,
                                RequestHandler,
                                config_directory,
                                config_sql_server,
-                               skip_db_cleanup,
                                package_data,
                                context,
                                check_env,
@@ -1116,7 +1195,8 @@ def start_server(config_directory, package_data, port, config_sql_server,
     atexit.register(unregister_handler, os.getpid())
 
     for _ in range(manager.worker_processes - 1):
-        p = Process(target=http_server.serve_forever)
+        # pylint: disable=no-member multiprocess module members.
+        p = multiprocess.Process(target=http_server.serve_forever)
         processes.append(p)
         p.start()
 
