@@ -18,9 +18,9 @@ import uuid
 from datetime import datetime
 from typing import Optional
 
-from codechecker_report_converter.util import load_json_or_empty
-
+from codechecker_common.compatibility.multiprocessing import cpu_count
 from codechecker_common.logger import get_logger
+from codechecker_common.util import load_json
 
 from codechecker_web.shared.env import check_file_owner_rw
 from codechecker_web.shared.version import SESSION_COOKIE_NAME as _SCN
@@ -61,7 +61,7 @@ def get_worker_processes(scfg_dict):
     Return 'worker_processes' field from the config dictionary or returns the
     default value if this field is not set or the value is negative.
     """
-    default = os.cpu_count()
+    default = cpu_count()
     worker_processes = scfg_dict.get('worker_processes', default)
 
     if worker_processes < 0:
@@ -262,8 +262,7 @@ class SessionManager:
         ValueError if the configuration file is invalid.
         """
         LOG.debug(self.__configuration_file)
-        cfg_dict = load_json_or_empty(self.__configuration_file, {},
-                                      'server configuration')
+        cfg_dict = load_json(self.__configuration_file, {})
         if cfg_dict != {}:
             check_file_owner_rw(self.__configuration_file)
         else:
